/****************************************************************************
** libmatroska : parse Matroska files, see http://www.matroska.org/
**
** <file/class description>
**
** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding an other license may use this file in accordance with 
** the Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.matroska.org/license/qpl/ for QPL licensing information.
** See http://www.matroska.org/license/gpl/ for GPL licensing information.
**
** Contact license@matroska.org if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

/*!
	\file
	\version \$Id: KaxCues.cpp,v 1.3 2003/05/10 17:26:41 robux4 Exp $
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include <cassert>

#include "StdInclude.h"
#include "KaxCues.h"
#include "KaxCuesData.h"
#include "KaxContexts.h"
#include "EbmlStream.h"

// sub elements
START_LIBMATROSKA_NAMESPACE

EbmlSemantic KaxCues_ContextList[1] = 
{
	EbmlSemantic(true,  false,  KaxCuePoint::ClassInfos),
};

const EbmlSemanticContext KaxCues_Context = EbmlSemanticContext(countof(KaxCues_ContextList), KaxCues_ContextList, &KaxSegment_Context, *GetKaxGlobal_Context, &KaxCues::ClassInfos);

EbmlId KaxCues_TheId(0x1C53BB6B, 4);
const EbmlCallbacks KaxCues::ClassInfos(KaxCues::Create, KaxCues_TheId, "Cues", KaxCues_Context);

KaxCues::KaxCues()
	:EbmlMaster()
{
	Context = &KaxCues_Context;
	ProcessMandatory();
}

KaxCues::~KaxCues()
{
	assert(myTempReferences.size() == 0); // otherwise that means you have added references and forgot to set the position
}

bool KaxCues::AddBlockGroup(const KaxBlockGroup & BlockReference)
{
	myTempReferences.push_back(&BlockReference);
	return true;
}

void KaxCues::PositionSet(const KaxBlockGroup & BlockReference)
{
	// look for the element in the temporary references
	std::list<const KaxBlockGroup *>::iterator ListIdx;

	for (ListIdx = myTempReferences.begin(); ListIdx != myTempReferences.end(); ListIdx++) {
		if (*ListIdx == &BlockReference) {
			// found, now add the element to the entry list
			KaxCuePoint & NewPoint = AddNewChild<KaxCuePoint>(*this);
			NewPoint.PositionSet(BlockReference);
			myTempReferences.erase(ListIdx);
			break;
		}
	}
}

/*!
	\warning Assume that the list has been sorted (Sort())
*/
uint64 KaxCues::GetTimecodePosition(uint64 aTimecode) const
{
	uint64 scale = 1000000; // fixed for the moment
	uint64 TimecodeToLocate = aTimecode / scale;
	const KaxCuePoint * aPointPrev = NULL;
	uint64 aPrevTime = 0;
	const KaxCuePoint * aPointNext = NULL;
	uint64 aNextTime = 0xFFFFFFFFFFFF;

	for (int i=0; i<ListSize(); i++)
	{
		if (EbmlId(*(*this)[i]) == KaxCuePoint::ClassInfos.GlobalId) {
			const KaxCuePoint *tmp = static_cast<const KaxCuePoint *>((*this)[i]);
			// check the tile
			const KaxCueTime *aTime = static_cast<const KaxCueTime *>(tmp->FindFirstElt(KaxCueTime::ClassInfos));
			if (aTime != NULL)
			{
				uint64 _Time = uint64(*aTime);
				if (_Time > aPrevTime && _Time < TimecodeToLocate) {
					aPrevTime = _Time;
					aPointPrev = tmp;
				}
				if (_Time < aNextTime && _Time > TimecodeToLocate) {
					aNextTime= _Time;
					aPointNext = tmp;
				}
			}
		}
	}
	if (aPointPrev != NULL)
	{
		uint64 aPosition = 0xFFFFFFFFFFFFFFF;
		// find the position of the "earlier" Cluster
		const KaxCueTrackPositions *aPoss = static_cast<const KaxCueTrackPositions *>(aPointPrev->FindFirstElt(KaxCueTrackPositions::ClassInfos));
		while (aPoss != NULL)
		{
			const KaxCueClusterPosition *aPos = static_cast<const KaxCueClusterPosition *>(aPoss->FindFirstElt(KaxCueClusterPosition::ClassInfos));
			if (aPos != NULL && uint64(*aPos) < aPosition)
				aPosition = uint64(*aPos);
			
			aPoss = static_cast<const KaxCueTrackPositions *>(aPointPrev->FindNextElt(*aPoss));
		}
		return aPosition;
	}

	return 0;
}

END_LIBMATROSKA_NAMESPACE
