/****************************************************************************
** libmatroska : parse Matroska files, see http://www.matroska.org/
**
** <file/class description>
**
** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
**
** This file is part of libmatroska.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding an other license may use this file in accordance with 
** the Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.matroska.org/license/qpl/ for QPL licensing information.
** See http://www.matroska.org/license/gpl/ for GPL licensing information.
**
** Contact license@matroska.org if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

/*!
	\file
	\version \$Id: KaxTag.cpp,v 1.4 2003/05/07 15:59:25 mosu Exp $
	\author Jory Stone       <jcsston @ toughguy.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.h"
#include "KaxTag.h"
#include "KaxTagMulti.h"
#include "KaxContexts.h"

using namespace LIBEBML_NAMESPACE;

// sub elements
START_LIBMATROSKA_NAMESPACE

EbmlSemantic KaxTag_ContextList[51] =
{	
	EbmlSemantic(true, false, KaxTagTrackUID::ClassInfos),
	EbmlSemantic(true, false, KaxTagChapterUID::ClassInfos),
	EbmlSemantic(true, false, KaxTagAbstract::ClassInfos),
	EbmlSemantic(true, false, KaxTagArchivalLocation::ClassInfos),
	EbmlSemantic(true, false, KaxTagAudioEncryption::ClassInfos),
	EbmlSemantic(true, false, KaxTagAudioGain::ClassInfos),
	EbmlSemantic(true, false, KaxTagAudioGenre::ClassInfos),
	EbmlSemantic(true, false, KaxTagAudioPeak::ClassInfos),
	EbmlSemantic(true, false, KaxTagAudioSecondaryGenre::ClassInfos),
	EbmlSemantic(true, false, KaxTagBibliography::ClassInfos),
	EbmlSemantic(true, false, KaxTagBPM::ClassInfos),
	EbmlSemantic(true, false, KaxTagCaptureDPI::ClassInfos),
	EbmlSemantic(true, false, KaxTagCapturePaletteSetting::ClassInfos),
	EbmlSemantic(true, false, KaxTagCropped::ClassInfos),
	EbmlSemantic(true, false, KaxTagDiscTrack::ClassInfos),
	EbmlSemantic(true, false, KaxTagEncoder::ClassInfos),
	EbmlSemantic(true, false, KaxTagEncodeSettings::ClassInfos),
	EbmlSemantic(true, false, KaxTagFile::ClassInfos),
	EbmlSemantic(true, false, KaxTagInitialKey::ClassInfos),
	EbmlSemantic(true, false, KaxTagKeywords::ClassInfos),
	EbmlSemantic(true, false, KaxTagLanguage::ClassInfos),
	EbmlSemantic(true, false, KaxTagLength::ClassInfos),
	EbmlSemantic(true, false, KaxTagMood::ClassInfos),
	EbmlSemantic(true, false, KaxTagOfficialAudioFileURL::ClassInfos),
	EbmlSemantic(true, false, KaxTagOfficialAudioSourceURL::ClassInfos),
	EbmlSemantic(true, false, KaxTagOriginalDimensions::ClassInfos),
	EbmlSemantic(true, false, KaxTagOriginalMediaType::ClassInfos),
	EbmlSemantic(true, false, KaxTagPlayCounter::ClassInfos),
	EbmlSemantic(true, false, KaxTagPlaylistDelay::ClassInfos),
	EbmlSemantic(true, false, KaxTagPopularimeter::ClassInfos),
	EbmlSemantic(true, false, KaxTagProduct::ClassInfos),
	EbmlSemantic(true, false, KaxTagRating::ClassInfos),
	EbmlSemantic(true, false, KaxTagRecordLocation::ClassInfos),
	EbmlSemantic(true, false, KaxTagSetPart::ClassInfos),
	EbmlSemantic(true, false, KaxTagSource::ClassInfos),
	EbmlSemantic(true, false, KaxTagSourceForm::ClassInfos),
	EbmlSemantic(true, false, KaxTagSubGenre::ClassInfos),
	EbmlSemantic(true, false, KaxTagSubject::ClassInfos),
	EbmlSemantic(true, false, KaxTagSynchronisedTempo::ClassInfos),
	EbmlSemantic(true, false, KaxTagSynchronisedText::ClassInfos),
	EbmlSemantic(true, false, KaxTagUnsynchronisedText::ClassInfos),
	EbmlSemantic(true, false, KaxTagUserDefinedURL::ClassInfos),
	EbmlSemantic(true, false, KaxTagVideoGenre::ClassInfos),
	EbmlSemantic(true, false, KaxTagVideoSecondaryGenre::ClassInfos),
	EbmlSemantic(false, true, KaxTagMultiComment::ClassInfos),
	EbmlSemantic(false, true, KaxTagMultiCommercial::ClassInfos),
	EbmlSemantic(false, true, KaxTagMultiDate::ClassInfos),
	EbmlSemantic(false, true, KaxTagMultiEntity::ClassInfos),
	EbmlSemantic(false, true, KaxTagMultiIdentifier::ClassInfos),
	EbmlSemantic(false, true, KaxTagMultiLegal::ClassInfos),
	EbmlSemantic(false, true, KaxTagMultiTitle::ClassInfos),
	//EbmlSemantic(true, false, KaxWritingApp::ClassInfos),
};

const EbmlSemanticContext KaxTag_Context = EbmlSemanticContext(countof(KaxTag_ContextList), KaxTag_ContextList, &KaxTags_Context, *GetKaxGlobal_Context, &KaxTag::ClassInfos);
const EbmlSemanticContext KaxTagTrackUID_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagTrackUID::ClassInfos);
const EbmlSemanticContext KaxTagChapterUID_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagChapterUID::ClassInfos);
const EbmlSemanticContext KaxTagAbstract_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagAbstract::ClassInfos);
const EbmlSemanticContext KaxTagArchivalLocation_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagArchivalLocation::ClassInfos);
const EbmlSemanticContext KaxTagAudioEncryption_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagAudioEncryption::ClassInfos);
const EbmlSemanticContext KaxTagAudioGain_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagAudioGain::ClassInfos);
const EbmlSemanticContext KaxTagAudioGenre_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagAudioGenre::ClassInfos);
const EbmlSemanticContext KaxTagAudioPeak_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagAudioPeak::ClassInfos);
const EbmlSemanticContext KaxTagAudioSecondaryGenre_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagAudioSecondaryGenre::ClassInfos);
const EbmlSemanticContext KaxTagBibliography_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagBibliography::ClassInfos);
const EbmlSemanticContext KaxTagBPM_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagBPM::ClassInfos);
const EbmlSemanticContext KaxTagCaptureDPI_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagCaptureDPI::ClassInfos);
const EbmlSemanticContext KaxTagCapturePaletteSetting_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagCapturePaletteSetting::ClassInfos);
const EbmlSemanticContext KaxTagCropped_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagCropped::ClassInfos);
const EbmlSemanticContext KaxTagDiscTrack_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagDiscTrack::ClassInfos);
const EbmlSemanticContext KaxTagEncoder_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagEncoder::ClassInfos);
const EbmlSemanticContext KaxTagEncodeSettings_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagEncodeSettings::ClassInfos);
const EbmlSemanticContext KaxTagFile_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagFile::ClassInfos);
const EbmlSemanticContext KaxTagInitialKey_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagInitialKey::ClassInfos);
const EbmlSemanticContext KaxTagKeywords_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagKeywords::ClassInfos);
const EbmlSemanticContext KaxTagLanguage_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagLanguage::ClassInfos);
const EbmlSemanticContext KaxTagLength_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagLength::ClassInfos);
const EbmlSemanticContext KaxTagMood_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMood::ClassInfos);
const EbmlSemanticContext KaxTagOfficialAudioFileURL_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagOfficialAudioFileURL::ClassInfos);
const EbmlSemanticContext KaxTagOfficialAudioSourceURL_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagOfficialAudioSourceURL::ClassInfos);
const EbmlSemanticContext KaxTagOriginalDimensions_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagOriginalDimensions::ClassInfos);
const EbmlSemanticContext KaxTagOriginalMediaType_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagOriginalMediaType::ClassInfos);
const EbmlSemanticContext KaxTagPlayCounter_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagPlayCounter::ClassInfos);
const EbmlSemanticContext KaxTagPlaylistDelay_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagPlaylistDelay::ClassInfos);
const EbmlSemanticContext KaxTagPopularimeter_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagPopularimeter::ClassInfos);
const EbmlSemanticContext KaxTagProduct_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagProduct::ClassInfos);
const EbmlSemanticContext KaxTagRating_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagRating::ClassInfos);
const EbmlSemanticContext KaxTagRecordLocation_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagRecordLocation::ClassInfos);
const EbmlSemanticContext KaxTagSetPart_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagSetPart::ClassInfos);
const EbmlSemanticContext KaxTagSource_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagSource::ClassInfos);
const EbmlSemanticContext KaxTagSourceForm_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagSourceForm::ClassInfos);
const EbmlSemanticContext KaxTagSubGenre_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagSubGenre::ClassInfos);
const EbmlSemanticContext KaxTagSubject_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagSubject::ClassInfos);
const EbmlSemanticContext KaxTagSynchronisedTempo_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagSynchronisedTempo::ClassInfos);
const EbmlSemanticContext KaxTagSynchronisedText_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagSynchronisedText::ClassInfos);
const EbmlSemanticContext KaxTagUnsynchronisedText_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagUnsynchronisedText::ClassInfos);
const EbmlSemanticContext KaxTagUserDefinedURL_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagUserDefinedURL::ClassInfos);
const EbmlSemanticContext KaxTagVideoGenre_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagVideoGenre::ClassInfos);
const EbmlSemanticContext KaxTagVideoSecondaryGenre_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagVideoSecondaryGenre::ClassInfos);
//const EbmlSemanticContext KaxMuxingApp_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxMuxingApp::ClassInfos);
//const EbmlSemanticContext KaxWritingApp_Context = EbmlSemanticContext(0, NULL, &KaxTag_Context, *GetKaxGlobal_Context, &KaxWritingApp::ClassInfos);

EbmlId KaxTag_TheId                      (0x7373, 2);
EbmlId KaxTagTrackUID_TheId              (0x73C5, 2);
EbmlId KaxTagChapterUID_TheId            (0x73C4, 2);
EbmlId KaxTagAbstract_TheId              (0x7300, 2);
EbmlId KaxTagArchivalLocation_TheId      (0x45A4, 2);
EbmlId KaxTagAudioEncryption_TheId       (0x87B4, 2);
EbmlId KaxTagAudioGain_TheId             (0x9999, 2);
EbmlId KaxTagAudioGenre_TheId            (0x12C2, 2);
EbmlId KaxTagAudioPeak_TheId             (0x9989, 2);
EbmlId KaxTagAudioSecondaryGenre_TheId   (0x87B1, 2);
EbmlId KaxTagBibliography_TheId          (0x4488, 2);
EbmlId KaxTagBPM_TheId                   (0x48A1, 2);
EbmlId KaxTagCaptureDPI_TheId            (0x82C7, 2);
EbmlId KaxTagCapturePaletteSetting_TheId (0xC334, 2);
EbmlId KaxTagCropped_TheId               (0xE387, 2);
EbmlId KaxTagDiscTrack_TheId             (0x5EB6, 2);
EbmlId KaxTagEncoder_TheId               (0xB431, 2);
EbmlId KaxTagEncodeSettings_TheId        (0xA526, 2);
EbmlId KaxTagFile_TheId                  (0xE54E, 2);
EbmlId KaxTagInitialKey_TheId            (0x153A, 2);
EbmlId KaxTagKeywords_TheId              (0x518C, 2);
EbmlId KaxTagLanguage_TheId              (0x22B59C, 3);
EbmlId KaxTagLength_TheId                (0x1243, 2);
EbmlId KaxTagMood_TheId                  (0x75AE, 2);
EbmlId KaxTagOfficialAudioFileURL_TheId	 (0x1C33, 2);
EbmlId KaxTagOfficialAudioSourceURL_TheId(0x4C3E, 2);
EbmlId KaxTagOriginalDimensions_TheId		 (0xE133, 2);
EbmlId KaxTagOriginalMediaType_TheId     (0x15A7, 2);
EbmlId KaxTagPlayCounter_TheId           (0x4566, 2);
EbmlId KaxTagPlaylistDelay_TheId         (0x32CC, 2);
EbmlId KaxTagPopularimeter_TheId         (0xCE32, 2);
EbmlId KaxTagProduct_TheId               (0x41E3, 2);
EbmlId KaxTagRating_TheId                (0x52BC, 2);
EbmlId KaxTagRecordLocation_TheId        (0x437E, 2);
EbmlId KaxTagSetPart_TheId               (0x336E, 2);
EbmlId KaxTagSource_TheId                (0x648A, 2);
EbmlId KaxTagSourceForm_TheId            (0x23B5, 2);
EbmlId KaxTagSubGenre_TheId              (0x62AC, 2);
EbmlId KaxTagSubject_TheId               (0x29C1, 2);
EbmlId KaxTagSynchronisedTempo_TheId     (0x1CAB, 2);
EbmlId KaxTagSynchronisedText_TheId      (0xBA59, 2);
EbmlId KaxTagUnsynchronisedText_TheId    (0x874B, 2);
EbmlId KaxTagUserDefinedURL_TheId        (0x234A, 2);
EbmlId KaxTagVideoGenre_TheId            (0x43A1, 2);
EbmlId KaxTagVideoSecondaryGenre_TheId   (0x43A2, 2);
//EbmlId KaxMuxingApp_TheId                (0x4D80, 2);
//EbmlId KaxWritingApp_TheId               (0x5741, 2);

const EbmlCallbacks KaxTag::ClassInfos(KaxTag::Create, KaxTag_TheId, "Tag", KaxTag_Context);
const EbmlCallbacks KaxTagTrackUID::ClassInfos(KaxTagTrackUID::Create, KaxTagTrackUID_TheId, "TagTrackUID", KaxTagTrackUID_Context);
const EbmlCallbacks KaxTagChapterUID::ClassInfos(KaxTagChapterUID::Create, KaxTagChapterUID_TheId, "TagChapterUID", KaxTagChapterUID_Context);
const EbmlCallbacks KaxTagAbstract::ClassInfos(KaxTagAbstract::Create, KaxTagAbstract_TheId, "KaxAbstract", KaxTagAbstract_Context);
const EbmlCallbacks KaxTagArchivalLocation::ClassInfos(KaxTagArchivalLocation::Create, KaxTagArchivalLocation_TheId, "ArchivalLocation", KaxTagArchivalLocation_Context);
const EbmlCallbacks KaxTagAudioEncryption::ClassInfos(KaxTagAudioEncryption::Create, KaxTagAudioEncryption_TheId, "AudioEncryption", KaxTagAudioEncryption_Context);
const EbmlCallbacks KaxTagAudioGain::ClassInfos(KaxTagAudioGain::Create, KaxTagAudioGain_TheId, "AudioGain", KaxTagAudioGain_Context);
const EbmlCallbacks KaxTagAudioGenre::ClassInfos(KaxTagAudioGenre::Create, KaxTagAudioGenre_TheId, "AudioGenre", KaxTagAudioGenre_Context);
const EbmlCallbacks KaxTagAudioPeak::ClassInfos(KaxTagAudioPeak::Create, KaxTagAudioPeak_TheId, "AudioPeak", KaxTagAudioPeak_Context);
const EbmlCallbacks KaxTagAudioSecondaryGenre::ClassInfos(KaxTagAudioSecondaryGenre::Create, KaxTagAudioSecondaryGenre_TheId, "AudioSecondaryGenre", KaxTagAudioSecondaryGenre_Context);
const EbmlCallbacks KaxTagBibliography::ClassInfos(KaxTagBibliography::Create, KaxTagBibliography_TheId, "Bibliography", KaxTagBibliography_Context);
const EbmlCallbacks KaxTagBPM::ClassInfos(KaxTagBPM::Create, KaxTagBPM_TheId, "BPM", KaxTagBPM_Context);
const EbmlCallbacks KaxTagCaptureDPI::ClassInfos(KaxTagCaptureDPI::Create, KaxTagCaptureDPI_TheId, "CaptureDPI", KaxTagCaptureDPI_Context);
const EbmlCallbacks KaxTagCapturePaletteSetting::ClassInfos(KaxTagCapturePaletteSetting::Create, KaxTagCapturePaletteSetting_TheId, "CapturePaletteSetting", KaxTagCapturePaletteSetting_Context);
const EbmlCallbacks KaxTagCropped::ClassInfos(KaxTagCropped::Create, KaxTagCropped_TheId, "Cropped", KaxTagCropped_Context);
const EbmlCallbacks KaxTagDiscTrack::ClassInfos(KaxTagDiscTrack::Create, KaxTagDiscTrack_TheId, "DiscTrack", KaxTagDiscTrack_Context);
const EbmlCallbacks KaxTagEncoder::ClassInfos(KaxTagEncoder::Create, KaxTagEncoder_TheId, "Encoder", KaxTagEncoder_Context);
const EbmlCallbacks KaxTagEncodeSettings::ClassInfos(KaxTagEncodeSettings::Create, KaxTagEncodeSettings_TheId, "EncodeSettings", KaxTagEncodeSettings_Context);
const EbmlCallbacks KaxTagFile::ClassInfos(KaxTagFile::Create, KaxTagEncodeSettings_TheId, "File", KaxTagFile_Context);
const EbmlCallbacks KaxTagInitialKey::ClassInfos(KaxTagInitialKey::Create, KaxTagInitialKey_TheId, "InitialKey", KaxTagInitialKey_Context);
const EbmlCallbacks KaxTagKeywords::ClassInfos(KaxTagKeywords::Create, KaxTagKeywords_TheId, "Keywords", KaxTagKeywords_Context);
const EbmlCallbacks KaxTagLanguage::ClassInfos(KaxTagLanguage::Create, KaxTagLanguage_TheId, "Language", KaxTagLanguage_Context);
const EbmlCallbacks KaxTagLength::ClassInfos(KaxTagLength::Create, KaxTagLength_TheId, "Length", KaxTagLength_Context);
const EbmlCallbacks KaxTagMood::ClassInfos(KaxTagMood::Create, KaxTagMood_TheId, "Mood", KaxTagMood_Context);
const EbmlCallbacks KaxTagOfficialAudioFileURL::ClassInfos(KaxTagOfficialAudioFileURL::Create, KaxTagOfficialAudioFileURL_TheId, "OfficialAudioFileURL", KaxTagOfficialAudioFileURL_Context);
const EbmlCallbacks KaxTagOfficialAudioSourceURL::ClassInfos(KaxTagOfficialAudioSourceURL::Create, KaxTagOfficialAudioSourceURL_TheId, "AudioSourceURL", KaxTagOfficialAudioSourceURL_Context);
const EbmlCallbacks KaxTagOriginalDimensions::ClassInfos(KaxTagOriginalDimensions::Create, KaxTagOriginalDimensions_TheId, "OriginalDimensions", KaxTagOriginalDimensions_Context);
const EbmlCallbacks KaxTagOriginalMediaType::ClassInfos(KaxTagOriginalMediaType::Create, KaxTagOriginalMediaType_TheId, "OriginalMediaType", KaxTagOriginalMediaType_Context);
const EbmlCallbacks KaxTagPlayCounter::ClassInfos(KaxTagPlayCounter::Create, KaxTagPlayCounter_TheId, "PlayCounter", KaxTagPlayCounter_Context);
const EbmlCallbacks KaxTagPlaylistDelay::ClassInfos(KaxTagPlaylistDelay::Create, KaxTagPlaylistDelay_TheId, "PlaylistDelay", KaxTagPlaylistDelay_Context);
const EbmlCallbacks KaxTagPopularimeter::ClassInfos(KaxTagPopularimeter::Create, KaxTagPopularimeter_TheId, "Popularimeter", KaxTagPopularimeter_Context);
const EbmlCallbacks KaxTagProduct::ClassInfos(KaxTagProduct::Create, KaxTagProduct_TheId, "Product", KaxTagProduct_Context);
const EbmlCallbacks KaxTagRating::ClassInfos(KaxTagRating::Create, KaxTagRating_TheId, "Rating", KaxTagRating_Context);
const EbmlCallbacks KaxTagRecordLocation::ClassInfos(KaxTagRecordLocation::Create, KaxTagRecordLocation_TheId, "RecordLocation", KaxTagRecordLocation_Context);
const EbmlCallbacks KaxTagSetPart::ClassInfos(KaxTagSetPart::Create, KaxTagSetPart_TheId, "SetPart", KaxTagSetPart_Context);
const EbmlCallbacks KaxTagSource::ClassInfos(KaxTagSource::Create, KaxTagSource_TheId, "Source", KaxTagSource_Context);
const EbmlCallbacks KaxTagSourceForm::ClassInfos(KaxTagSourceForm::Create, KaxTagSourceForm_TheId, "SourceForm", KaxTagSourceForm_Context);
const EbmlCallbacks KaxTagSubGenre::ClassInfos(KaxTagSubGenre::Create, KaxTagSubGenre_TheId, "SubGenre", KaxTagSubGenre_Context);
const EbmlCallbacks KaxTagSubject::ClassInfos(KaxTagSubject::Create, KaxTagSubject_TheId, "Subject", KaxTagSubject_Context);
const EbmlCallbacks KaxTagSynchronisedTempo::ClassInfos(KaxTagSynchronisedTempo::Create, KaxTagSynchronisedTempo_TheId, "SynchronisedTempo", KaxTagSynchronisedTempo_Context);
const EbmlCallbacks KaxTagSynchronisedText::ClassInfos(KaxTagSynchronisedText::Create, KaxTagSynchronisedText_TheId, "SynchronisedText", KaxTagSynchronisedText_Context);
const EbmlCallbacks KaxTagUnsynchronisedText::ClassInfos(KaxTagUnsynchronisedText::Create, KaxTagUnsynchronisedText_TheId, "UnsynchronisedText", KaxTagUnsynchronisedText_Context);
const EbmlCallbacks KaxTagUserDefinedURL::ClassInfos(KaxTagUserDefinedURL::Create, KaxTagUserDefinedURL_TheId, "UserDefinedURL", KaxTagUserDefinedURL_Context);
const EbmlCallbacks KaxTagVideoGenre::ClassInfos(KaxTagVideoGenre::Create, KaxTagVideoGenre_TheId, "VideoGenre", KaxTagVideoGenre_Context);
const EbmlCallbacks KaxTagVideoSecondaryGenre::ClassInfos(KaxTagVideoSecondaryGenre::Create, KaxTagVideoSecondaryGenre_TheId, "VideoSecondaryGenre", KaxTagVideoSecondaryGenre_Context);
//const EbmlCallbacks KaxMuxingApp::ClassInfos(KaxMuxingApp::Create, KaxMuxingApp_TheId, "MuxingApp", KaxMuxingApp_Context);
//const EbmlCallbacks KaxWritingApp::ClassInfos(KaxWritingApp::Create, KaxWritingApp_TheId, "WritingApp", KaxWritingApp_Context);

KaxTag::KaxTag()
	:EbmlMaster()
{
	Context = &KaxTag_Context;
	ProcessMandatory();
}

END_LIBMATROSKA_NAMESPACE
