/****************************************************************************
** libmatroska : parse Matroska files, see http://www.matroska.org/
**
** <file/class description>
**
** Copyright (C) 2002-2003 Steve Lhomme.  All rights reserved.
**
** This file is part of libmatroska.
**
** This file may be distributed under the terms of the Q Public License
** as defined by Trolltech AS of Norway and appearing in the file
** LICENSE.QPL included in the packaging of this file.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE.GPL included in the
** packaging of this file.
**
** Licensees holding an other license may use this file in accordance with 
** the Agreement provided with the Software.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://www.matroska.org/license/qpl/ for QPL licensing information.
** See http://www.matroska.org/license/gpl/ for GPL licensing information.
**
** Contact license@matroska.org if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

/*!
	\file
	\version \$Id: KaxTagMulti.cpp,v 1.6 2003/05/07 15:59:25 mosu Exp $
	\author Jory Stone       <jcsston @ toughguy.net>
	\author Steve Lhomme     <robux4 @ users.sf.net>
*/
#include "StdInclude.h"
#include "KaxTagMulti.h"
#include "KaxContexts.h"

using namespace LIBEBML_NAMESPACE;

START_LIBMATROSKA_NAMESPACE

EbmlSemantic KaxTagMultiComment_ContextList[2] =
{
	EbmlSemantic(true, false, KaxTagMultiCommentComments::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiCommentLanguage::ClassInfos),
};

EbmlSemantic KaxTagMultiCommercial_ContextList[5] =
{
	EbmlSemantic(true, false, KaxTagMultiCommercialType::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiCommercialAddress::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiCommercialURL::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiCommercialEmail::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiPrice::ClassInfos),
};

EbmlSemantic KaxTagMultiPrice_ContextList[3] =
{
	EbmlSemantic(true, false, KaxTagMultiPriceCurrency::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiPriceAmount::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiPricePriceDate::ClassInfos),
};

EbmlSemantic KaxTagMultiDate_ContextList[3] =
{
	EbmlSemantic(true, false, KaxTagMultiDateType::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiDateDateBegin::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiDateDateEnd::ClassInfos),
};

EbmlSemantic KaxTagMultiEntity_ContextList[5] =
{
	EbmlSemantic(true, false, KaxTagMultiEntityType::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiEntityName::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiEntityURL::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiEntityEmail::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiEntityAddress::ClassInfos),
};

EbmlSemantic KaxTagMultiIdentifier_ContextList[3] =
{
	EbmlSemantic(true, false, KaxTagMultiIdentifierType::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiIdentifierBinary::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiIdentifierString::ClassInfos),
};

EbmlSemantic KaxTagMultiLegal_ContextList[3] =
{
	EbmlSemantic(true, false, KaxTagMultiLegalType::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiLegalURL::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiLegalAddress::ClassInfos),
};

EbmlSemantic KaxTagMultiTitle_ContextList[8] =
{
	EbmlSemantic(true, false, KaxTagMultiTitleType::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiTitleName::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiTitleSubTitle::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiTitleEdition::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiTitleAddress::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiTitleURL::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiTitleEmail::ClassInfos),
	EbmlSemantic(true, false, KaxTagMultiTitleLanguage::ClassInfos),
};

const EbmlSemanticContext KaxTagMultiComment_Context = EbmlSemanticContext(countof(KaxTagMultiComment_ContextList), KaxTagMultiComment_ContextList, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMultiComment::ClassInfos);
const EbmlSemanticContext KaxTagMultiCommentComments_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiComment_Context, *GetKaxGlobal_Context, &KaxTagMultiCommentComments::ClassInfos);
const EbmlSemanticContext KaxTagMultiCommentLanguage_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiComment_Context, *GetKaxGlobal_Context, &KaxTagMultiCommentLanguage::ClassInfos);

const EbmlSemanticContext KaxTagMultiCommercial_Context = EbmlSemanticContext(countof(KaxTagMultiCommercial_ContextList), KaxTagMultiCommercial_ContextList, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMultiCommercial::ClassInfos);
const EbmlSemanticContext KaxTagMultiCommercialType_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiCommercial_Context, *GetKaxGlobal_Context, &KaxTagMultiCommercialType::ClassInfos);
const EbmlSemanticContext KaxTagMultiCommercialAddress_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiCommercial_Context, *GetKaxGlobal_Context, &KaxTagMultiCommercialAddress::ClassInfos);
const EbmlSemanticContext KaxTagMultiCommercialURL_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiCommercial_Context, *GetKaxGlobal_Context, &KaxTagMultiCommercialURL::ClassInfos);
const EbmlSemanticContext KaxTagMultiCommercialEmail_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiCommercial_Context, *GetKaxGlobal_Context, &KaxTagMultiCommercialEmail::ClassInfos);

const EbmlSemanticContext KaxTagMultiPrice_Context = EbmlSemanticContext(countof(KaxTagMultiPrice_ContextList), KaxTagMultiPrice_ContextList, &KaxTagMultiCommercial_Context, *GetKaxGlobal_Context, &KaxTagMultiPrice::ClassInfos);
const EbmlSemanticContext KaxTagMultiPriceCurrency_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiPrice_Context, *GetKaxGlobal_Context, &KaxTagMultiPriceCurrency::ClassInfos);
const EbmlSemanticContext KaxTagMultiPriceAmount_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiPrice_Context, *GetKaxGlobal_Context, &KaxTagMultiPriceAmount::ClassInfos);
const EbmlSemanticContext KaxTagMultiPricePriceDate_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiPrice_Context, *GetKaxGlobal_Context, &KaxTagMultiPricePriceDate::ClassInfos);

const EbmlSemanticContext KaxTagMultiDate_Context = EbmlSemanticContext(countof(KaxTagMultiDate_ContextList), KaxTagMultiDate_ContextList, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMultiDate::ClassInfos);
const EbmlSemanticContext KaxTagMultiDateType_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiDate_Context, *GetKaxGlobal_Context, &KaxTagMultiDateType::ClassInfos);
const EbmlSemanticContext KaxTagMultiDateDateBegin_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiDate_Context, *GetKaxGlobal_Context, &KaxTagMultiDateDateBegin::ClassInfos);
const EbmlSemanticContext KaxTagMultiDateDateEnd_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiDate_Context, *GetKaxGlobal_Context, &KaxTagMultiDateDateEnd::ClassInfos);

const EbmlSemanticContext KaxTagMultiEntity_Context = EbmlSemanticContext(countof(KaxTagMultiEntity_ContextList), KaxTagMultiEntity_ContextList, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMultiEntity::ClassInfos);
const EbmlSemanticContext KaxTagMultiEntityType_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiEntity_Context, *GetKaxGlobal_Context, &KaxTagMultiEntityType::ClassInfos);
const EbmlSemanticContext KaxTagMultiEntityName_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiEntity_Context, *GetKaxGlobal_Context, &KaxTagMultiEntityName::ClassInfos);
const EbmlSemanticContext KaxTagMultiEntityURL_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiEntity_Context, *GetKaxGlobal_Context, &KaxTagMultiEntityURL::ClassInfos);
const EbmlSemanticContext KaxTagMultiEntityEmail_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiEntity_Context, *GetKaxGlobal_Context, &KaxTagMultiEntityEmail::ClassInfos);
const EbmlSemanticContext KaxTagMultiEntityAddress_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiEntity_Context, *GetKaxGlobal_Context, &KaxTagMultiEntityAddress::ClassInfos);

const EbmlSemanticContext KaxTagMultiIdentifier_Context = EbmlSemanticContext(countof(KaxTagMultiIdentifier_ContextList), KaxTagMultiIdentifier_ContextList, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMultiIdentifier::ClassInfos);
const EbmlSemanticContext KaxTagMultiIdentifierType_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiIdentifier_Context, *GetKaxGlobal_Context, &KaxTagMultiIdentifierType::ClassInfos);
const EbmlSemanticContext KaxTagMultiIdentifierBinary_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiIdentifier_Context, *GetKaxGlobal_Context, &KaxTagMultiIdentifierBinary::ClassInfos);
const EbmlSemanticContext KaxTagMultiIdentifierString_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiIdentifier_Context, *GetKaxGlobal_Context, &KaxTagMultiIdentifierString::ClassInfos);

const EbmlSemanticContext KaxTagMultiLegal_Context = EbmlSemanticContext(countof(KaxTagMultiLegal_ContextList), KaxTagMultiLegal_ContextList, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMultiLegal::ClassInfos);
const EbmlSemanticContext KaxTagMultiLegalType_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiLegal_Context, *GetKaxGlobal_Context, &KaxTagMultiLegalType::ClassInfos);
const EbmlSemanticContext KaxTagMultiLegalURL_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiLegal_Context, *GetKaxGlobal_Context, &KaxTagMultiLegalURL::ClassInfos);
const EbmlSemanticContext KaxTagMultiLegalAddress_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiLegal_Context, *GetKaxGlobal_Context, &KaxTagMultiLegalAddress::ClassInfos);

const EbmlSemanticContext KaxTagMultiTitle_Context = EbmlSemanticContext(countof(KaxTagMultiTitle_ContextList), KaxTagMultiTitle_ContextList, &KaxTag_Context, *GetKaxGlobal_Context, &KaxTagMultiTitle::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleType_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleType::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleName_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleName::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleSubTitle_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleSubTitle::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleEdition_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleEdition::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleAddress_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleAddress::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleURL_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleURL::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleEmail_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleEmail::ClassInfos);
const EbmlSemanticContext KaxTagMultiTitleLanguage_Context = EbmlSemanticContext(0, NULL, &KaxTagMultiTitle_Context, *GetKaxGlobal_Context, &KaxTagMultiTitleLanguage::ClassInfos);


//The Muti Elements
EbmlId KaxTagMultiCommercial_TheId          (0x4DC7, 2);
EbmlId KaxTagMultiCommercialType_TheId      (0x5BD7, 2);
EbmlId KaxTagMultiCommercialAddress_TheId   (0x5BDB, 2);
EbmlId KaxTagMultiCommercialURL_TheId       (0x5BDA, 2);
EbmlId KaxTagMultiCommercialEmail_TheId     (0x5BC0, 2);

EbmlId KaxTagMultiPrice_TheId               (0x5BC3, 2);
EbmlId KaxTagMultiPriceCurrency_TheId       (0x6C, 1);
EbmlId KaxTagMultiPriceAmount_TheId         (0x6E, 1);
EbmlId KaxTagMultiPricePriceDate_TheId      (0x6F, 1);

EbmlId KaxTagMultiDate_TheId                (0x4DC8, 2);
EbmlId KaxTagMultiDateType_TheId            (0x5BD8, 2);
EbmlId KaxTagMultiDateDateBegin_TheId       (0x4461, 2);
EbmlId KaxTagMultiDateDateEnd_TheId         (0x4462, 2);

EbmlId KaxTagMultiEntity_TheId              (0x4DC9, 2);
EbmlId KaxTagMultiEntityType_TheId          (0x5BD9, 2);
EbmlId KaxTagMultiEntityName_TheId          (0x5BED, 2);
EbmlId KaxTagMultiEntityAddress_TheId       (0x5BDC, 2);
EbmlId KaxTagMultiEntityURL_TheId           (0x5BDB, 2);
EbmlId KaxTagMultiEntityEmail_TheId         (0x5BC1, 2);

EbmlId KaxTagMultiIdentifier_TheId          (0x4DC6, 2);
EbmlId KaxTagMultiIdentifierType_TheId      (0x5BDA, 2);
EbmlId KaxTagMultiIdentifierBinary_TheId    (0x6B67, 2);
EbmlId KaxTagMultiIdentifierString_TheId    (0x6B68, 2);

EbmlId KaxTagMultiLegal_TheId               (0x4DC5, 2);
EbmlId KaxTagMultiLegalType_TheId           (0x5BDB, 2);
EbmlId KaxTagMultiLegalURL_TheId            (0x5B34, 2);
EbmlId KaxTagMultiLegalAddress_TheId        (0x5BB9, 2);

EbmlId KaxTagMultiTitle_TheId               (0x4DC4, 2);
EbmlId KaxTagMultiTitleType_TheId           (0x5B7D, 2);
EbmlId KaxTagMultiTitleName_TheId           (0x5BB9, 2);
EbmlId KaxTagMultiTitleSubTitle_TheId       (0x5B5B, 2);
EbmlId KaxTagMultiTitleEdition_TheId        (0x5BAE, 2);
EbmlId KaxTagMultiTitleAddress_TheId        (0x5B33, 2);
EbmlId KaxTagMultiTitleURL_TheId            (0x5BA9, 2);
EbmlId KaxTagMultiTitleEmail_TheId          (0x5BC9, 2);
EbmlId KaxTagMultiTitleLanguage_TheId       (0x22B59C, 3);

EbmlId KaxTagMultiComment_TheId             (0x5B7B, 2);
EbmlId KaxTagMultiCommentComments_TheId     (0x5F7C, 2);
EbmlId KaxTagMultiCommentLanguage_TheId     (0x22B59D, 3);

const EbmlCallbacks KaxTagMultiComment::ClassInfos(KaxTagMultiComment::Create, KaxTagMultiComment_TheId, "MultiComment", KaxTagMultiComment_Context);
const EbmlCallbacks KaxTagMultiCommentComments::ClassInfos(KaxTagMultiCommentComments::Create, KaxTagMultiCommentComments_TheId, "MultiCommentComments", KaxTagMultiCommentComments_Context);
const EbmlCallbacks KaxTagMultiCommentLanguage::ClassInfos(KaxTagMultiCommentLanguage::Create, KaxTagMultiCommentLanguage_TheId, "MultiCommentLanguage", KaxTagMultiCommentLanguage_Context);

const EbmlCallbacks KaxTagMultiCommercial::ClassInfos(KaxTagMultiCommercial::Create, KaxTagMultiCommercial_TheId, "MultiCommercial", KaxTagMultiCommercial_Context);
const EbmlCallbacks KaxTagMultiCommercialType::ClassInfos(KaxTagMultiCommercialType::Create, KaxTagMultiCommercialType_TheId, "MultiCommercialType", KaxTagMultiCommercialType_Context);
const EbmlCallbacks KaxTagMultiCommercialAddress::ClassInfos(KaxTagMultiCommercialAddress::Create, KaxTagMultiCommercialAddress_TheId, "MultiCommercialAddress", KaxTagMultiCommercialAddress_Context);
const EbmlCallbacks KaxTagMultiCommercialURL::ClassInfos(KaxTagMultiCommercialURL::Create, KaxTagMultiCommercialURL_TheId, "MultiCommercialURL", KaxTagMultiCommercialURL_Context);
const EbmlCallbacks KaxTagMultiCommercialEmail::ClassInfos(KaxTagMultiCommercialEmail::Create, KaxTagMultiCommercialEmail_TheId, "MultiCommercialEmail", KaxTagMultiCommercialEmail_Context);

const EbmlCallbacks KaxTagMultiPrice::ClassInfos(KaxTagMultiPrice::Create, KaxTagMultiPrice_TheId, "MultiPrice", KaxTagMultiPrice_Context);
const EbmlCallbacks KaxTagMultiPriceCurrency::ClassInfos(KaxTagMultiPriceCurrency::Create, KaxTagMultiPriceCurrency_TheId, "MultiPriceCurrency", KaxTagMultiPriceCurrency_Context);
const EbmlCallbacks KaxTagMultiPriceAmount::ClassInfos(KaxTagMultiPriceAmount::Create, KaxTagMultiPriceAmount_TheId, "MultiPriceAmount", KaxTagMultiPriceAmount_Context);
const EbmlCallbacks KaxTagMultiPricePriceDate::ClassInfos(KaxTagMultiPricePriceDate::Create, KaxTagMultiPricePriceDate_TheId, "MultiPricePriceDate", KaxTagMultiPricePriceDate_Context);

const EbmlCallbacks KaxTagMultiDate::ClassInfos(KaxTagMultiDate::Create, KaxTagMultiDate_TheId, "MultiDate", KaxTagMultiDate_Context);
const EbmlCallbacks KaxTagMultiDateType::ClassInfos(KaxTagMultiDateType::Create, KaxTagMultiDateType_TheId, "MultiDateType", KaxTagMultiDateType_Context);
const EbmlCallbacks KaxTagMultiDateDateBegin::ClassInfos(KaxTagMultiDateDateBegin::Create, KaxTagMultiDateDateBegin_TheId, "MultiDateDateBegin", KaxTagMultiDateDateBegin_Context);
const EbmlCallbacks KaxTagMultiDateDateEnd::ClassInfos(KaxTagMultiDateDateEnd::Create, KaxTagMultiDateDateEnd_TheId, "MultiDateDateEnd", KaxTagMultiDateDateEnd_Context);

const EbmlCallbacks KaxTagMultiEntity::ClassInfos(KaxTagMultiEntity::Create, KaxTagMultiEntity_TheId, "MultiEntity", KaxTagMultiEntity_Context);
const EbmlCallbacks KaxTagMultiEntityType::ClassInfos(KaxTagMultiEntityType::Create, KaxTagMultiEntityType_TheId, "MultiEntityType", KaxTagMultiEntityType_Context);
const EbmlCallbacks KaxTagMultiEntityName::ClassInfos(KaxTagMultiEntityName::Create, KaxTagMultiEntityName_TheId, "MultiEntityName", KaxTagMultiEntityName_Context);
const EbmlCallbacks KaxTagMultiEntityURL::ClassInfos(KaxTagMultiEntityURL::Create, KaxTagMultiEntityURL_TheId, "MultiEntityURL", KaxTagMultiEntityURL_Context);
const EbmlCallbacks KaxTagMultiEntityEmail::ClassInfos(KaxTagMultiEntityEmail::Create, KaxTagMultiEntityEmail_TheId, "MultiEntityEmail", KaxTagMultiEntityEmail_Context);
const EbmlCallbacks KaxTagMultiEntityAddress::ClassInfos(KaxTagMultiEntityAddress::Create, KaxTagMultiEntityAddress_TheId, "MultiEntityAddress", KaxTagMultiEntityAddress_Context);

const EbmlCallbacks KaxTagMultiIdentifier::ClassInfos(KaxTagMultiIdentifier::Create, KaxTagMultiIdentifier_TheId, "MultiIdentifier", KaxTagMultiIdentifier_Context);
const EbmlCallbacks KaxTagMultiIdentifierType::ClassInfos(KaxTagMultiDateType::Create, KaxTagMultiDateType_TheId, "KaxTagMultiIdentifierType", KaxTagMultiIdentifierType_Context);
const EbmlCallbacks KaxTagMultiIdentifierBinary::ClassInfos(KaxTagMultiIdentifierBinary::Create, KaxTagMultiIdentifierBinary_TheId, "MultiIdentifierBinary", KaxTagMultiIdentifierBinary_Context);
const EbmlCallbacks KaxTagMultiIdentifierString::ClassInfos(KaxTagMultiIdentifierString::Create, KaxTagMultiIdentifierString_TheId, "MultiIdentifierString", KaxTagMultiIdentifierString_Context);

const EbmlCallbacks KaxTagMultiLegal::ClassInfos(KaxTagMultiLegal::Create, KaxTagMultiLegal_TheId, "MultiLegal", KaxTagMultiLegal_Context);
const EbmlCallbacks KaxTagMultiLegalType::ClassInfos(KaxTagMultiLegalType::Create, KaxTagMultiLegalType_TheId, "KaxTagMultiLegalType", KaxTagMultiLegalType_Context);
const EbmlCallbacks KaxTagMultiLegalURL::ClassInfos(KaxTagMultiLegalURL::Create, KaxTagMultiLegalURL_TheId, "KaxTagMultiLegalURL", KaxTagMultiLegalURL_Context);
const EbmlCallbacks KaxTagMultiLegalAddress::ClassInfos(KaxTagMultiLegalAddress::Create, KaxTagMultiLegalAddress_TheId, "KaxTagMultiLegalAddress", KaxTagMultiLegalAddress_Context);

const EbmlCallbacks KaxTagMultiTitle::ClassInfos(KaxTagMultiTitle::Create, KaxTagMultiTitle_TheId, "MultiEntity", KaxTagMultiTitle_Context);
const EbmlCallbacks KaxTagMultiTitleType::ClassInfos(KaxTagMultiTitleType::Create, KaxTagMultiTitleType_TheId, "MultiTitleType", KaxTagMultiTitleType_Context);
const EbmlCallbacks KaxTagMultiTitleName::ClassInfos(KaxTagMultiTitleName::Create, KaxTagMultiTitleName_TheId, "MultiTitleName", KaxTagMultiTitleName_Context);
const EbmlCallbacks KaxTagMultiTitleSubTitle::ClassInfos(KaxTagMultiTitleSubTitle::Create, KaxTagMultiTitleSubTitle_TheId, "MultiTitleSubTitle", KaxTagMultiTitleSubTitle_Context);
const EbmlCallbacks KaxTagMultiTitleEdition::ClassInfos(KaxTagMultiTitleEdition::Create, KaxTagMultiTitleEdition_TheId, "MultiTitleEdition", KaxTagMultiTitleEdition_Context);
const EbmlCallbacks KaxTagMultiTitleAddress::ClassInfos(KaxTagMultiTitleAddress::Create, KaxTagMultiTitleAddress_TheId, "MultiTitleAddress", KaxTagMultiTitleAddress_Context);
const EbmlCallbacks KaxTagMultiTitleURL::ClassInfos(KaxTagMultiTitleURL::Create, KaxTagMultiTitleURL_TheId, "MultiTitleURL", KaxTagMultiTitleURL_Context);
const EbmlCallbacks KaxTagMultiTitleEmail::ClassInfos(KaxTagMultiTitleEmail::Create, KaxTagMultiTitleEmail_TheId, "MultiTitleEmail", KaxTagMultiTitleEmail_Context);
const EbmlCallbacks KaxTagMultiTitleLanguage::ClassInfos(KaxTagMultiTitleLanguage::Create, KaxTagMultiTitleLanguage_TheId, "MultiTitleLanguage", KaxTagMultiTitleLanguage_Context);

KaxTagMultiComment::KaxTagMultiComment()
	:EbmlMaster()
{
	Context = &KaxTagMultiComment_Context;
	ProcessMandatory();
}


KaxTagMultiCommercial::KaxTagMultiCommercial()
	:EbmlMaster()
{
	Context = &KaxTagMultiCommercial_Context;
	ProcessMandatory();
}

KaxTagMultiPrice::KaxTagMultiPrice()
	:EbmlMaster()
{
	Context = &KaxTagMultiPrice_Context;
	ProcessMandatory();
}

KaxTagMultiDate::KaxTagMultiDate()
	:EbmlMaster()
{
	Context = &KaxTagMultiDate_Context;
	ProcessMandatory();
}

KaxTagMultiEntity::KaxTagMultiEntity()
	:EbmlMaster()
{
	Context = &KaxTagMultiEntity_Context;
	ProcessMandatory();
}
	
KaxTagMultiLegal::KaxTagMultiLegal()
	:EbmlMaster()
{
	Context = &KaxTagMultiLegal_Context;
	ProcessMandatory();
}

KaxTagMultiIdentifier::KaxTagMultiIdentifier()
	:EbmlMaster()
{
	Context = &KaxTagMultiIdentifier_Context;
	ProcessMandatory();
}

KaxTagMultiTitle::KaxTagMultiTitle()
	:EbmlMaster()
{
	Context = &KaxTagMultiTitle_Context;
	ProcessMandatory();
}

END_LIBMATROSKA_NAMESPACE
