/*
    Copyright (C) 2003, Jory Stone <jcsston @ toughguy.net>
		Copyright (C) 2003, John Cannon <spyder482 @ yahoo.com>
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2mka-wxgui.

    Mpa2mka-wxgui is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Mpa2mka-wxgui is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Mpa2mka-wxgui; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file mpa2mka-wxgui.cpp
    wxWindows GUI to Convert WAV or MPEG I/II Layer 1/2/3 to Matroska files
*/

#include "mpa2mka-wxgui.h"

#define mpa2mka_wxgui_app_name "mpa2mka-wxgui v0.1.3"
using namespace LIBMATROSKA_NAMESPACE;
using namespace LIBEBML_NAMESPACE;
using namespace std;

AppFrame *frame;
// program execution "starts" here
bool MainApp::OnInit()
{
    frame = new AppFrame(_T(mpa2mka_wxgui_app_name), wxPoint(-1, -1), wxSize(434, 320));
    frame->Show(TRUE);

    return TRUE;
}

// frame constructor
AppFrame::AppFrame(const wxString &title, const wxPoint &pos, const wxSize& size) : wxFrame ((wxFrame *) NULL, -1, title, pos, size)
{
	mpa_convert_object = NULL;
	//Load & set the frame icon
	wxIcon frame_ico;
	frame_ico.LoadFile(_T("dialogs"), wxBITMAP_TYPE_ICO_RESOURCE);
	SetIcon(frame_ico);

	// create menus
	fileMenu = new wxMenu(_T(""), wxMENU_TEAROFF);
	fileMenu->Append(MainApp_On_inputfile_browse_Button, _T("Locate &Input Audio File\tCtrl-I"), _T("Open Input MPEG Audio File (mp1, mp2, mp3) or a WAV file"));
	fileMenu->Append(MainApp_On_outputfile_browse_Button, _T("Locate &Output Mka File\tCtrl-O"), _T("Open Output Matroska (mka) file"));
	fileMenu->AppendSeparator();
	fileMenu->Append(MainApp_Quit, _T("E&xit\tCtrl-Q"), _T("Quit this program"));
	optionsMenu = new wxMenu;
	optionsMenu->Append(MainApp_OptionsBackgroundColor, _T("&Background Color\tCtrl-B"), _T("Choose the background color"));
	optionsMenu->AppendCheckItem(MainApp_OptionsDragnDrop, _T("&Auto convert Drag-n-Drop Files\tCtrl-A"), _T("Converts files without asking, appending .mka to the output filename"));
	helpMenu = new wxMenu;
	helpMenu->Append(MainApp_About, _T("&About...\tCtrl-A"), _T("Show about dialog"));

	menuBar = new wxMenuBar();
	menuBar->Append(fileMenu, _T("&File"));
	menuBar->Append(optionsMenu, _T("&Options"));
	menuBar->Append(helpMenu, _T("&Help"));

	SetMenuBar(menuBar);

	verbose_output = new wxCheckBox(this, MainApp_Verbose_Checkbox, _T("Verbose Output"), wxPoint(14,98), wxSize(100,20));
	verbose_output->SetToolTip(_T("Please tell me you know what this does"));
	disable_lacing = new wxCheckBox(this, -1, _T("Disable Lacing"), wxPoint(118,98), wxSize(108,20));
	disable_lacing->SetToolTip(_T("Disable Lacing in the Matroska output file file. Results in larger files"));
	copy_id3tags = new wxCheckBox(this, -1, _T("Copy ID3 Tags"), wxPoint(226,98), wxSize(110,20));
	copy_id3tags->SetToolTip(_T("This only applies to mpa's"));
	mpa2mka_output_text = new wxTextCtrl(this, -1, _T(""), wxPoint(8, 120), wxSize(296, 112), wxTE_RICH|wxTE_MULTILINE|wxTE_READONLY);
	mpa2mka_output_text->SetToolTip(_T("The displays the progress of the conversion\nAlso a drop target"));
	wxTextAttr default_txt_style = mpa2mka_output_text->GetDefaultStyle();
	mpa2mka_output_text->SetDefaultStyle(wxTextAttr(*wxBLUE));
	mpa2mka_output_text->AppendText(_T("Status Infomation:\n"));
	mpa2mka_output_text->SetDefaultStyle(default_txt_style);
	mpa2mka_output_text->SetInsertionPoint(mpa2mka_output_text->GetLastPosition());
	dnd_load = new DnDFile(mpa2mka_output_text);
	mpa2mka_output_text->SetDropTarget(dnd_load);


	start_button = new wxButton(this, MainApp_On_start_Button, _T("Start"), wxPoint(344, 16), wxSize(72, 24));
	start_button->SetToolTip(_T("Start Conversion"));    
	close_button = new wxButton(this, MainApp_On_close_Button, _T("Close"), wxPoint(344, 48), wxSize(72, 24));
	close_button->SetToolTip(_T("Now why would you exit?"));    
	about_button = new wxButton(this, MainApp_About, _T("About"), wxPoint(344, 82), wxSize(72, 24));
	about_button->SetToolTip(_T("Yet Another About Box"));

	inputfile_browse_button = new wxButton(this, MainApp_On_inputfile_browse_Button, _T("..."), wxPoint(304, 26), wxSize(24, 24));
	inputfile_browse_button->SetToolTip(_T("Browse for a audio input file"));
	outputfile_browse_button = new wxButton(this, MainApp_On_outputfile_browse_Button, _T("..."), wxPoint(304, 72), wxSize(24, 24));
	outputfile_browse_button->SetToolTip(_T("Browse for the Matroska output file"));

	inputfile_text = new wxTextCtrl(this, -1, _T(""),wxPoint(8, 26), wxSize(296, 24));
	inputfile_text->SetToolTip(_T("Enter the Intput Audio Filename here"));
	inputfile_label = new wxStaticText(this, -1, _T("Input MPA File:"), wxPoint(16,8), wxSize(288, 16));
	outputfile_text = new wxTextCtrl(this, -1, _T(""),wxPoint(8, 72), wxSize(296, 24));
	outputfile_text->SetToolTip(_T("Enter the Output Matroska Filename here"));
	outputfile_label = new wxStaticText(this, -1, _T("Output MKA File"), wxPoint(16, 56), wxSize(288, 16));

	max_cluster_label = new wxStaticText(this, -1, _T("Max Cluster"), wxPoint(304, 120),wxSize(104,16), wxALIGN_CENTRE|wxST_NO_AUTORESIZE);    
	max_cluster_size_label = new wxStaticText(this, -1,_T(" In Size (kB)"), wxPoint(320, 136), wxSize(88, 16));    		
	max_cluster_size_text = new wxSpinCtrl(this, MainApp_Cluster_Text, _T(""), wxPoint(320, 152), wxSize(72, 24), wxSP_ARROW_KEYS, 1, 10240, 512);
	max_cluster_size_text->SetToolTip(_T("Default Size is 512kB"));		
	max_cluster_time_label = new wxStaticText(this, -1,_T(" In Time (ms)"), wxPoint(320, 184), wxSize(80,16));				
	max_cluster_time_text = new wxSpinCtrl(this, MainApp_Cluster_Text, _T(""), wxPoint(320, 200), wxSize(72, 24), wxSP_ARROW_KEYS, 1, 60000, 5000);
	max_cluster_time_text->SetToolTip(_T("Default Time is 5000ms"));

	//Load User Settings
	wxConfig *pConfig = new wxConfig(_T("mpa2mka-wxgui"));    

	int backgroundColor_r, backgroundColor_g, backgroundColor_b;
	wxColor custom_background;
	wxString maxCluster_str;

	pConfig->Read(_T("/Colors/Background_R"), &backgroundColor_r, 250);
	pConfig->Read(_T("/Colors/Background_G"), &backgroundColor_g, 250);
	pConfig->Read(_T("/Colors/Background_B"), &backgroundColor_b, 250);    		
	custom_background.Set(backgroundColor_r, backgroundColor_g, backgroundColor_b);		
	UpdateColors(custom_background);

	maxCluster_str.Printf(_T("%i"), pConfig->Read(_T("/Cluster/maxSize"), 512));
	max_cluster_size_text->SetValue(maxCluster_str);
	maxCluster_str.Printf(_T("%i"), pConfig->Read(_T("/Cluster/maxTime"), 5000));
	max_cluster_time_text->SetValue(maxCluster_str);

	bool saved_check;
	pConfig->Read(_T("/Verbose"), &saved_check, false);
	verbose_output->SetValue(saved_check);
	pConfig->Read(_T("/Lacing Disabled"), &saved_check, false);
	disable_lacing->SetValue(saved_check );
	pConfig->Read(_T("/Copy ID3 Tags"), &saved_check, true);
	copy_id3tags->SetValue(saved_check);
	pConfig->Read(_T("/Auto DragnDrop"), &saved_check, true);
	optionsMenu->FindItem(MainApp_OptionsDragnDrop)->Check(saved_check);
	delete pConfig;
	    
	// create a statusbar
	CreateStatusBar(1);
	SetStatusText(_T("Ready."));
}


/*****************
 * event handlers
 *****************/

void
AppFrame::OnQuit(wxCommandEvent &event)
{
    // TRUE is to force the frame to close
    Close(TRUE);
}

void
AppFrame::OnOpen(wxCommandEvent &event)
{
  OnInputFileBrowseButton(event);
}

void AppFrame::OnAbout(wxCommandEvent &event)
{
    wxString msg;

    msg = _T(mpa2mka_wxgui_app_name " converts an Audio File to a Matroska file.\n");
    msg += _T("(c) 2003 Jory Stone <jcsston@toughguy.net> - GPL License\n");
		msg += _T("(c) 2003 John Cannon <spyder482@yahoo.com> - GPL License\n");
    msg += _T("Portions (c) 2002 Steve Lhomme - GPL License\nhttp://www.matroska.org/\n");
		msg += _T("Versions\n" mpa2mka_app_name " " wav2mka_app_name);
    wxMessageBox(msg, _T("About"), wxOK | wxICON_INFORMATION, this);
}

void AppFrame::OnOkButton	(wxCommandEvent &event)
{
  wxString msg = _T("Nothing");
  wxMessageBox(msg, _T("t"), wxOK | wxICON_INFORMATION, this);   
}

void AppFrame::OnStartButton(wxCommandEvent &event)
{
  wxString input_file, output_file;

  if (wxFileExists(inputfile_text->GetValue())) {
    input_file = inputfile_text->GetValue();    
    if (outputfile_text->GetValue().Len() > 1) {
      output_file = outputfile_text->GetValue();
    }else {
      output_file = (inputfile_text->GetValue() + _T(".mka"));
    }    
    if (wxFileExists(output_file)) {
      wxString msg;
      msg = _T("Output file: \"") + output_file + _T("\"\nAlready exists! Overwrite?");
      wxMessageDialog *overwrite = new wxMessageDialog(this, msg, _T("Overwrite?"), wxYES_NO|wxCENTER);
      if (overwrite->ShowModal() == wxID_NO)
        return;
    }		
    //We passed all the checks, now convert!
		if (input_file.Right(3).Upper() == "WAV") {
			wxTextAttr default_txt_style = mpa2mka_output_text->GetDefaultStyle();
			mpa2mka_output_text->SetDefaultStyle(wxTextAttr(*wxRED));
			mpa2mka_output_text->AppendText(_T("wav2mka Status Infomation:\n"));
			mpa2mka_output_text->SetDefaultStyle(default_txt_style);
			mpa2mka_output_text->SetInsertionPoint(mpa2mka_output_text->GetLastPosition());

			WAV2MKA *convert_object;
			convert_object = new WAV2MKA;

			convert_object->SetVerboseModeOn(verbose_output->IsChecked());
			convert_object->SetLacingOff(disable_lacing->IsChecked());
			convert_object->SetMaxClusterTimeSize(max_cluster_time_text->GetValue());
			convert_object->SetMaxClusterByteSize((max_cluster_size_text->GetValue() * 1024));
			convert_object->ConvertWAV2Matroska_wxWindows(mpa2mka_output_text, (std::string)input_file.mb_str(), (std::string)output_file.mb_str());
			delete convert_object;
		}else {
			wxTextAttr default_txt_style = mpa2mka_output_text->GetDefaultStyle();
			mpa2mka_output_text->SetDefaultStyle(wxTextAttr(*wxBLUE));
			mpa2mka_output_text->AppendText(_T("mpa2mka Status Infomation:\n"));
			mpa2mka_output_text->SetDefaultStyle(default_txt_style);
			mpa2mka_output_text->SetInsertionPoint(mpa2mka_output_text->GetLastPosition());
			
			MPA2MKA *convert_object;
			convert_object = new MPA2MKA;
			mpa_convert_object = convert_object;
			
			convert_object->SetVerboseModeOn(verbose_output->IsChecked());
			convert_object->SetLacingOff(disable_lacing->IsChecked());
			convert_object->SetTagCopying(copy_id3tags->IsChecked());
			convert_object->SetMaxClusterTimeSize(max_cluster_time_text->GetValue());
			convert_object->SetMaxClusterByteSize((max_cluster_size_text->GetValue() * 1024));
			convert_object->ConvertMPA2Matroska_wxWindows(mpa2mka_output_text, (std::string)input_file.mb_str(), (std::string)output_file.mb_str());		
			delete convert_object;
			mpa_convert_object = NULL;
		}

		//Save the settings used
		wxConfig *pConfig = new wxConfig(_T("mpa2mka-wxgui"));
		pConfig->Write(_T("/Cluster/maxSize"), max_cluster_size_text->GetValue());
		pConfig->Write(_T("/Cluster/maxTime"), max_cluster_time_text->GetValue());
		pConfig->Write(_T("/Verbose"), verbose_output->IsChecked());
		pConfig->Write(_T("/Lacing Disabled"), disable_lacing->IsChecked());
		pConfig->Write(_T("/Copy ID3 Tags"), copy_id3tags->IsChecked());
		pConfig->Write(_T("/Auto DragnDrop"), frame->optionsMenu->FindItem(MainApp_OptionsDragnDrop)->IsChecked());
		delete pConfig;

	}else {
    wxString msg;
    msg = _T("Input file: \"") + inputfile_text->GetValue() + _T("\"\nDoes not exist!");
    wxMessageBox(msg,_T("Input file doesn't exist"));
  }
}

void AppFrame::OnCloseButton(wxCommandEvent &event)
{
  OnQuit(event);
}

void AppFrame::OnInputFileBrowseButton(wxCommandEvent &event)
{
  wxString allowed_file_types;
	//Setup allowed file types
	allowed_file_types = _T("Audio files|*.wav;*.mpa;*.mp1;*.mp2;*.mp3|");
	allowed_file_types += _T("MPEG Audio files (*.mpa;*.mp1;*.mp2;*.mp3)|*.mpa;*.mp1;*.mp2;*.mp3|");
	allowed_file_types += _T("WAV files (*.wav)|*.wav|");
	allowed_file_types += _T("All files (*.*)|*.*");

	wxFileDialog dialog (this, _T("Browse for input audio file"), _T(""), inputfile_text->GetValue(), allowed_file_types, wxHIDE_READONLY);

  if (dialog.ShowModal() == wxID_OK) {
	  inputfile_text->SetValue(dialog.GetPath());
    if (outputfile_text->GetLineLength(0) < 2) {
      outputfile_text->SetValue(inputfile_text->GetValue() + _T(".mka"));
    }
  }
	
}

void AppFrame::OnOutputFileBrowseButton(wxCommandEvent &event)
{
  wxFileDialog dialog (this, _T("Browse for output Matroska file"), _T(""), outputfile_text->GetValue(), _T("Matroska Audio files (*.mka)|*.mka"), wxSAVE);

  if (dialog.ShowModal() == wxID_OK) {
	  outputfile_text->SetValue(dialog.GetPath());
  }
}
void AppFrame::OnCheck_VerboseCheckbox(wxCommandEvent &event)
{
	if (mpa_convert_object)
		mpa_convert_object->SetVerboseModeOn(verbose_output->IsChecked());
}

void AppFrame::OptionsBackgroundColor(wxCommandEvent &event)
{
  //Let the user choose the background color for the window
  wxColourData data;
  data.SetChooseFull(TRUE);
  for (int i = 0; i < 16; i++)
  {
    wxColour colour(i*16, i*16, i*16);		
    data.SetCustomColour(i, colour);
  }
	wxConfig *pConfig = new wxConfig(_T("mpa2mka-wxgui"));

	int backgroundColor_r, backgroundColor_g, backgroundColor_b;
	wxColor custom_background;
	pConfig->Read(_T("/Colors/Background_R"), &backgroundColor_r);
	pConfig->Read(_T("/Colors/Background_G"), &backgroundColor_g);
	pConfig->Read(_T("/Colors/Background_B"), &backgroundColor_b);

	custom_background.Set(backgroundColor_r, backgroundColor_g, backgroundColor_b);
	data.SetColour(custom_background);

  wxColourDialog *dialog = new wxColourDialog(this, &data);
  if (dialog->ShowModal() == wxID_OK)
  {
    wxColourData retData = dialog->GetColourData();
    wxColor col = retData.GetColour();
    
    pConfig->Write(_T("/Colors/Background_R"), int(col.Red()));
    pConfig->Write(_T("/Colors/Background_G"), int(col.Green()));
    pConfig->Write(_T("/Colors/Background_B"), int(col.Blue()));

    UpdateColors(col);
  }
  dialog->Destroy();
	delete pConfig;
}

void AppFrame::UpdateColors(wxColor background)
{
  //Update the background color
  wxWindow::SetBackgroundColour(background);
  wxWindow::Refresh();  
	verbose_output->SetBackgroundColour(background);
  verbose_output->Refresh();
  disable_lacing->SetBackgroundColour(background);
  disable_lacing->Refresh();
	copy_id3tags->SetBackgroundColour(background);
  copy_id3tags->Refresh();
	max_cluster_label->SetBackgroundColour(background);
  max_cluster_label->Refresh();
  max_cluster_size_label->SetBackgroundColour(background);
  max_cluster_size_label->Refresh();
  max_cluster_time_label->SetBackgroundColour(background);
  max_cluster_time_label->Refresh();
  inputfile_label->SetBackgroundColour(background);
  inputfile_label->Refresh();
  outputfile_label->SetBackgroundColour(background);
  outputfile_label->Refresh();

}

void AppFrame::OptionsDragnDrop (wxCommandEvent &event)
{
	//
}

bool DnDFile::OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames)
{
  wxString dnd_file;
  unsigned int i;

	if (filenames.GetCount() < 2 || (!frame->optionsMenu->FindItem(MainApp_OptionsDragnDrop)->IsChecked())) {
		frame->inputfile_text->SetValue(filenames[0]);
		frame->outputfile_text->SetValue(filenames[0] + _T(".mka"));
	}else {
		for(i=0;i<filenames.GetCount();i++) {
			dnd_file = filenames[i];
			if (dnd_file.Right(3).Upper() == "WAV") {
				wxString status_text;
				status_text.Printf(_T("%i of %i : Converting WAV to Matroska"), i, filenames.GetCount());
				frame->SetStatusText(status_text);
			
				wxTextAttr default_txt_style = frame->mpa2mka_output_text->GetDefaultStyle();
				frame->mpa2mka_output_text->SetDefaultStyle(wxTextAttr(*wxRED));
				frame->mpa2mka_output_text->AppendText(_T("wav2mka Status Infomation:\n"));
				frame->mpa2mka_output_text->SetDefaultStyle(default_txt_style);
				frame->mpa2mka_output_text->SetInsertionPoint(frame->mpa2mka_output_text->GetLastPosition());

				WAV2MKA *convert_object;
				convert_object = new WAV2MKA;
				
				convert_object->SetVerboseModeOn(frame->verbose_output->IsChecked());
				convert_object->SetLacingOff(frame->disable_lacing->IsChecked());
				convert_object->SetMaxClusterTimeSize(frame->max_cluster_time_text->GetValue());
				convert_object->SetMaxClusterByteSize((frame->max_cluster_size_text->GetValue() * 1024));
				convert_object->ConvertWAV2Matroska_wxWindows(frame->mpa2mka_output_text, (std::string)dnd_file.mb_str());
				delete convert_object;
				frame->SetStatusText(_T("Done"));
			}
			else {
				wxString status_text;
				status_text.Printf(_T("%i of %i : Converting MPA to Matroska"), i, filenames.GetCount());
				frame->SetStatusText(status_text);

				wxTextAttr default_txt_style = frame->mpa2mka_output_text->GetDefaultStyle();
				frame->mpa2mka_output_text->SetDefaultStyle(wxTextAttr(*wxBLUE));
				frame->mpa2mka_output_text->AppendText(_T("mpa2mka Status Infomation:\n"));
				frame->mpa2mka_output_text->SetDefaultStyle(default_txt_style);
				frame->mpa2mka_output_text->SetInsertionPoint(frame->mpa2mka_output_text->GetLastPosition());


				//MPA2MKA *convert_object;
				frame->mpa_convert_object = new MPA2MKA;
				
				frame->mpa_convert_object->SetVerboseModeOn(frame->verbose_output->IsChecked());
				frame->mpa_convert_object->SetLacingOff(frame->disable_lacing->IsChecked());
				frame->mpa_convert_object->SetTagCopying(frame->copy_id3tags->IsChecked());
				frame->mpa_convert_object->SetMaxClusterTimeSize(frame->max_cluster_time_text->GetValue());
				frame->mpa_convert_object->SetMaxClusterByteSize((frame->max_cluster_size_text->GetValue() * 1024));
				frame->mpa_convert_object->ConvertMPA2Matroska_wxWindows(frame->mpa2mka_output_text, (std::string)dnd_file.mb_str());
				delete frame->mpa_convert_object;
				frame->SetStatusText(_T("Done"));
			}
		}
	}
  return true;
}