#ifndef _MAIN_H_
#define _MAIN_H_

#include <wx/wx.h>
#include <wx/image.h>
#include <wx/grid.h>
#include <wx/spinbutt.h>
#include <wx/spinctrl.h>
#include <wx/splitter.h>
#include <wx/statline.h>
#include <wx/tglbtn.h>
#include <wx/notebook.h>
#include <wx/mdi.h>
#include "wx/busyinfo.h"
#include "wx/filedlg.h"
#include "wx/colordlg.h"
#include "wx/generic/colrdlgg.h"
#include "wx/config.h"
#include "wx/confbase.h"
#include "wx/dnd.h"

#include <errno.h>
#include <ctype.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
//#include <unistd.h>

#ifdef GCC2
#include <typeinfo>
#endif

#include <iostream>

#ifdef DMALLOC
#include <dmalloc.h>
#endif

#include "mpa2mka.h"
#include "wav2mka.h"

class DnDFile : public wxFileDropTarget
{
  public:
    DnDFile(wxTextCtrl *pOwner) { m_pOwner = pOwner; }

    virtual bool OnDropFiles(wxCoord x, wxCoord y, const wxArrayString& filenames);

  private:
    wxTextCtrl *m_pOwner;
};

class MainApp : public wxApp
{
  public:
    virtual bool OnInit(void);
};

class AppFrame : public wxFrame
{
  public:
    AppFrame(const wxString &title, const wxPoint &pos, const wxSize &size);

    // event handlers
		void OnQuit(wxCommandEvent &event);
		void OnOpen(wxCommandEvent &event);
    void OptionsBackgroundColor(wxCommandEvent &event);
		void OptionsDragnDrop (wxCommandEvent &event);
    void OnAbout(wxCommandEvent &event);
		void OnOkButton(wxCommandEvent &event);
    void OnStartButton(wxCommandEvent &event);
    void OnCloseButton(wxCommandEvent &event);
    void OnInputFileBrowseButton(wxCommandEvent &event);
    void OnOutputFileBrowseButton(wxCommandEvent &event);
		void OnCheck_VerboseCheckbox(wxCommandEvent &event);
    void UpdateColors(wxColor background);
		
  //protected:
    // controls
		MPA2MKA *mpa_convert_object;
		DnDFile *dnd_load;
		wxMenuBar *menuBar;
    wxMenu *fileMenu;
    wxMenu *optionsMenu;
    wxMenu *helpMenu;
    wxButton *start_button;
		wxButton *close_button;
		wxButton *about_button;
		wxButton *inputfile_browse_button;
		wxButton *outputfile_browse_button;
		wxCheckBox *verbose_output;
		wxCheckBox *disable_lacing;
		wxCheckBox *copy_id3tags;
    wxTextCtrl *mpa2mka_output_text;
    wxTextCtrl *inputfile_text;
    wxTextCtrl *outputfile_text;
    wxSpinCtrl *max_cluster_size_text;
    wxSpinCtrl *max_cluster_time_text;
    wxStaticText *inputfile_label;
    wxStaticText *outputfile_label;
    wxStaticText *max_cluster_label;
    wxStaticText *max_cluster_size_label;
    wxStaticText *max_cluster_time_label;
				
  private:
    // any class wishing to process wxWindows events must use this macro
    DECLARE_EVENT_TABLE()
};


enum
{
    // menu items
    MainApp_Quit = 1,
		MainApp_Open,
    MainApp_OptionsBackgroundColor,
		MainApp_OptionsDragnDrop,
    MainApp_About,
		MainApp_On_start_Button,
    MainApp_On_close_Button,
    MainApp_On_inputfile_browse_Button,
    MainApp_On_outputfile_browse_Button,
		MainApp_Cluster_Text,
		MainApp_Verbose_Checkbox
};


/**********************************************************
 * Event tables and other macros for wxWindows
 **********************************************************
 * The event tables connect the wxWindows events with the
 * functions (event handlers) which process them. It can
 * be also done at run-time, but for the simple menu
 * events like this the static method is much simpler.
 *
 **********************************************************/

BEGIN_EVENT_TABLE(AppFrame, wxFrame)
    EVT_MENU(MainApp_Quit, AppFrame::OnQuit)
		EVT_MENU(MainApp_Open, AppFrame::OnOpen)
    EVT_MENU(MainApp_OptionsBackgroundColor, AppFrame::OptionsBackgroundColor)
		EVT_MENU(MainApp_OptionsDragnDrop, AppFrame::OptionsDragnDrop)
    EVT_MENU(MainApp_About, AppFrame::OnAbout)
    EVT_BUTTON(MainApp_On_start_Button, AppFrame::OnStartButton)
    EVT_BUTTON(MainApp_On_close_Button, AppFrame::OnCloseButton)
    EVT_BUTTON(MainApp_About, AppFrame::OnAbout)
    EVT_BUTTON(MainApp_On_inputfile_browse_Button, AppFrame::OnInputFileBrowseButton)
    EVT_BUTTON(MainApp_On_outputfile_browse_Button, AppFrame::OnOutputFileBrowseButton)
		EVT_CHECKBOX(MainApp_Verbose_Checkbox, AppFrame::OnCheck_VerboseCheckbox)
END_EVENT_TABLE()


/*****************************************************************
 * Create a new application object: this macro will allow
 * wxWindows to create the application object during program
 * execution (it's better than using a static object for many
 * reasons) and also declares the accessor function wxGetApp()
 * which will return the reference of the right type (i.e. MyApp
 * and not wxApp)
 *
 *****************************************************************/

IMPLEMENT_APP(MainApp)

#endif _MAIN_H__