/*
    Copyright (C) 2003, John Cannon <spyder482 @ yahoo.com>
    Copyright (C) 2003, Jory Stone <jcsston @ toughguy.net>
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2mka.

    Mpa2mka is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Mpa2mka is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file main.cpp
    Convert MPEG I/II Layer 1/2/3 to Matroska files
    \todo remove bad frames    
*/

#include "mpa2mka.h"

void usage()
{
  printf("%s - (c) 2003 John Cannon & Jory Stone - GPL License\n", mpa2mka_app_name);
  printf("Portions (c) 2002 Steve Lhomme - GPL License\n");
	printf("This program converts a MPEG-I/II Layer 1,2,3 file into a Matroska file\r\n\r\n");
  printf("Usage: mpa2mka [options] <input-filename> [output-filename]\n");
  printf("\tif no output file is specified the input filename is appended with .mka\r\n\r\n");
	printf("\tOptions:\r\n\r\n");
	printf("\t\t -v VERBOSE\tDisplay more infomation.\n\n");
	printf("\t\t -nl NO LACING\tTurns off lacing.\n\n");
	printf("\t\t -c SIZE\tSets the max size in kB of a cluster.\n\t\t\t\tDefault is 512kB\n\n");
	printf("\t\t -t TIME\tSets the max time in ms per cluster.\n\t\t\t\tDefault is 10000ms\n\n");
			
}

int main(int argc,char *argv[])
{
	using namespace std;
	int cl_offset = 0;
	bool bVerbose = false;
	bool noLacing = false;
	uint32 maxTimePerCluster = 5000; //10 seconds
	uint64 maxClusterSize = 512 * 1024; //512kB

	// check for the valid number of parameters
  if (argc < 2 || argc > 8) {
		usage();
		return -1;
  }

	if(argc > 2){ //option may be included
		for(int i = 0; i < 4; i++){
			if((argv[1+cl_offset][0] == '-') && ((argv[1+cl_offset][1] == 'c') || (argv[1+cl_offset][1] == 't') || (argv[1+cl_offset][1] == 'v') || ((argv[1+cl_offset][1] == 'n') && (argv[1+cl_offset][2] == 'l')))){
				if(argv[1+cl_offset][1] == 'c'){
					maxClusterSize = (atoi(argv[2+cl_offset]) * 1024);
					cl_offset+=2;
				}
				if(argv[1+cl_offset][1] == 't'){
					uint32 maxTimePerCluster = atoi(argv[2+cl_offset]);
					cl_offset+=2;
				}
				if(argv[1+cl_offset][1] == 'v'){
					bVerbose = true;
					
					cl_offset+=1;
				}
				if((argv[1+cl_offset][1] == 'n') && (argv[1+cl_offset][2] == 'l')){
					noLacing = true;
					
					cl_offset+=1;
				}
				//cl_offset+=2;
			}else{
				break;
			}
		}
	}
  std::string output_Filename;

	if (argc > cl_offset+2) {
    output_Filename = argv[2+cl_offset];
	}else {
    output_Filename.append(argv[1+cl_offset]);
		output_Filename.append(".mka");
	}
	MPA2MKA *new_convert_process;
	new_convert_process = new MPA2MKA();

	new_convert_process->SetVerboseModeOn(bVerbose);
	new_convert_process->SetLacingOff(noLacing);
	new_convert_process->SetMaxClusterTimeSize(maxTimePerCluster);
	new_convert_process->SetMaxClusterByteSize(maxClusterSize);
	cout << new_convert_process->ConvertMPA2Matroska(argv[1+cl_offset], output_Filename);

	return 0;
}