/*
    Copyright (C) 2003, John Cannon <spyder482 @ yahoo.com>
    Copyright (C) 2003, Jory Stone <jcsston @ toughguy.net>
    Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
    This file is part of mpa2mka.

    Mpa2mka is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Mpa2mka is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Foobar; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
    \file mpa2mka.cpp
    Convert MPEG I/II Layer 1/2/3 to Matroska files
    \todo remove bad frames    
*/
//Comment this out if you don't want ID3 tag support
#define HAVE_ID3LIB

#include "mpa2mka.h"

using namespace LIBMATROSKA_NAMESPACE;
using namespace LIBEBML_NAMESPACE;
using namespace std;

#if defined wxUSE_GUI
#define printf OutputToBuffer
void MPA2MKA::ConvertMPA2Matroska_wxWindows(wxTextCtrl *target_console_output_box, std::string passed_InputFile, std::string passed_OutputFile)
{
	console_output_box = target_console_output_box;
	ConvertMPA2Matroska(passed_InputFile, passed_OutputFile);
	return;
}
#endif

MPA2MKA::MPA2MKA()
{
	currentBlockGroup = NULL;
	clust = NULL;
	data = NULL;

	currentBlockGroupFull = false;
	noLacing = false;
	bVerbose = false;
	bCopyTags = true;
	maxTimePerCluster = 5000; //10 seconds
	maxClusterSize = 512 * 1024; //512kB
	currentClusterSize = 0;
	currentClusterBaseTimecode = 0;
	currentClusterBlockTimecode = 0;
	TIMECODE_SCALE = 1000000;
	srand((unsigned)time(NULL));
	track_uid = rand();
};

MPA2MKA::~MPA2MKA()
{
	//Free memory here
	//I need to know how to free memory used by libmatroska objects
	//delete frame_buffer;
	delete data;
};

bool MPA2MKA::SetInputFile(std::string passed_InputFile)
{
	input_Filename = passed_InputFile;
	return true;
};

bool MPA2MKA::SetOutputFile(std::string passed_OutputFile)
{
	output_Filename = passed_OutputFile;
	return true;
};

bool MPA2MKA::SetVerboseModeOn(bool bVerboseOn)
{
	bVerbose = bVerboseOn;
	return true;
};

bool MPA2MKA::SetLacingOff(bool bLacingOff)
{
	noLacing = bLacingOff;
	return true;
};

bool MPA2MKA::SetTagCopying(bool bTagCopying)
{
	bCopyTags = bTagCopying;
	return true;
}

bool MPA2MKA::SetMaxClusterByteSize(uint64 passed_maxClusterSize)
{
	//The allowed range is 128 bytes to 1073741824 bytes or 1GB
	if ((passed_maxClusterSize > 128) && (passed_maxClusterSize < 1073741824)) {
		maxClusterSize = passed_maxClusterSize;
		return true;
	}else {
		//Assign the default value
		maxClusterSize = 512 * 1024;
		return false;
	}
};

bool MPA2MKA::SetMaxClusterTimeSize(uint32 passed_maxTimePerCluster)
{
	//The allowed range is 10ms to 3600000ms or 1 hour
	if ((passed_maxTimePerCluster > 10) && (passed_maxTimePerCluster < 3600000)) {
		maxTimePerCluster = passed_maxTimePerCluster;
		return true;
	}else {
		//Assign the default value, 5 seconds
		maxTimePerCluster = 5000;
		return false;
	}
};

void MPA2MKA::OutputToBuffer(char *formating_string, ...)
{
	va_list argptr;

	//Get a pointer to the list of arg
	va_start(argptr, formating_string);	
#if defined wxUSE_GUI 
	//Redirect the output to a wxWindow wxTextCtrl
	wxString formated_text;
	formated_text.PrintfV(wxString(formating_string, wxConvUTF8).c_str(), argptr);
	console_output_box->AppendText(formated_text);
	//Give the processor some relief
	wxYield();
#else
	//Just let the function call pass through unaffected
	vprintf(formating_string, argptr);	
#endif
}

int MPA2MKA::ConvertMPA2Matroska(std::string passed_InputFile, std::string passed_OutputFile)
{
	using namespace M2W_NAMESPACE;
	//Store values
	input_Filename = passed_InputFile;

	if (passed_OutputFile == "")
		output_Filename = passed_InputFile + ".mka";
	else
		output_Filename = passed_OutputFile;

	try{
		printf("%s - (c) 2003 John Cannon & Jory Stone\n", mpa2mka_app_name);
		printf("Portions (c) 2002 Steve Lhomme\n");
		printf("Under the GNU GPL License\n\n");
		
		if (bVerbose)
			printf("In Verbose Mode, prepare to be blasted with numbers\n");
		if (noLacing)
			printf("Lacing turned off\n");

    // open the input file		
    MPA_File input_File(input_Filename, MODE_READ);
    
    StdIOCallback output_file(output_Filename.c_str(), MODE_CREATE);

	
    // prepare to read the whole file for statistics
  	if (bVerbose){
      printf("Reading statistics in %s\n", input_Filename.c_str());
	  }

    input_File.SkipToFrameStart();

    MPA_Lame_Frame infoFrame;
    bool bInfoFrameOn = false;
    if (input_File.CheckInfoFrame()) {
		  *static_cast<MPA_Frame *>(&infoFrame) = input_File.GetInfoFrame();
		  bInfoFrameOn = true;
    }

    uint32 FrameNumber = 0;
    uint16 Layer = 0;
    uint64 SampleNumbers = 0;
    uint64 TotalFrameSize = 0;
    uint16 SamplingRate = 0;
    uint16 Channels = 0;
    uint16 SampleSize = 0;
    MPA_Frame frame;
    
    while (input_File.FindNextSyncWord()) {
		  FrameNumber++;
		  frame = input_File.GetFrame();
		  /// \todo handle the bitrate of the frame in a key/value table, so that we can detect real CBR files
		  TotalFrameSize += frame.DataSize() + 4;
		  SampleNumbers += frame.SampleSize();
		  SamplingRate = frame.SamplingRate();
		  Channels = frame.Channels();
		  SampleSize = frame.SampleSize();
		  Layer = frame.Layer();
		  input_File.SkipData();
    }

    uint64 totalByteSize = TotalFrameSize * SamplingRate;
    uint32 AverageBitrate, AverageByterate;
    if ((8 * totalByteSize) % SampleNumbers > SampleNumbers>>1)
	    AverageBitrate = 1 + 8 * totalByteSize / SampleNumbers;
    else
	    AverageBitrate = 8 * totalByteSize / SampleNumbers;

    if (AverageBitrate % 8 > 4)
	    AverageByterate = 1 + (AverageBitrate >> 3);
    else
	    AverageByterate = AverageBitrate >> 3;

	  printf("Maximum Cluster size is %dkB\n", (maxClusterSize/1024));
	  printf("Maximum time per cluster is %dms\n", maxTimePerCluster);

		Duration_total = (((int64)SampleNumbers * 1000) / SamplingRate);
    InitMatroskaOutputFile(output_file, SamplingRate, Layer, Channels);
    
    ChangeClusters(output_file, 0);
		currentBlockGroup  = &clust->GetNewBlock();

    // prepare to read for output
    input_File.SkipToFrameStart();

    /// \todo add the option to skip the Info frame or not
	  if (bInfoFrameOn) {
		  input_File.FindNextSyncWord();
		  input_File.GetFrame();
		  input_File.SkipData();
    }
    
	  if (bVerbose) {	
		  printf("Writing frames in %s\n", passed_OutputFile.c_str());
	  }
    uint32 prevSamplesNumber = 0;
    SampleNumbers = 0;
    int i=0;
    while (input_File.FindNextSyncWord()) {
		  //binary frame_buffer[5000+4]; // should be enough for 32kHz/320kbps
      frame_buffer = new binary[5000+4];
	    frame = input_File.GetFrame();
	    SampleNumbers += frame.SampleSize();
	    memcpy(frame_buffer, (const binary *)(frame), 4);
	    input_File.GetData(frame_buffer+4, 5000);
	    /// \todo handle timecode
      if (bVerbose) {
        printf("frame %4d : samples %d ", i++, SampleNumbers);
        printf("in pos %d ", input_File.getFilePointer());
        printf("size %d\n", frame.DataSize()+4);
		  }
		  AddFrame(output_file, MyTrack1, frame_buffer, frame.DataSize()+4, (((uint64)prevSamplesNumber * 1000) / SamplingRate));
		  currentClusterSize += frame.DataSize()+4;		
		  prevSamplesNumber = SampleNumbers;
    }			
    CloseMatroskaFile(output_file);

    output_file.close();
    input_File.close();

		currentBlockGroupFull = false;
		currentClusterSize = 0;
		currentClusterBaseTimecode = 0;

		printf("\n");
    return 0;    
    }catch (exception & Ex){
        cout << Ex.what() << endl;
        return -1;
    }
}

int MPA2MKA::InitMatroskaOutputFile(StdIOCallback & output_file, uint16 SamplingRate, uint16 Layer, uint16 Channels){
	try{
		// write output file
		EbmlHead FileHead;
	  
		EDocType & MyDocType = GetChild<EDocType>(FileHead);
		*static_cast<EbmlString *>(&MyDocType) = "matroska";
	  
		EDocTypeVersion & MyDocTypeVer = GetChild<EDocTypeVersion>(FileHead);
		*(static_cast<EbmlUInteger *>(&MyDocTypeVer)) = 1;
	  
		EDocTypeReadVersion & MyDocTypeReadVer = GetChild<EDocTypeReadVersion>(FileHead);
		*(static_cast<EbmlUInteger *>(&MyDocTypeReadVer)) = 1;
	  
		FileHead.Render(output_file);
	  
		
		uint64 SegmentSize = FileSegment.Render(output_file);

		Dummy.SetSize(100); // 100 octets
		Dummy.Render(output_file);

		//Add the Info segment
		KaxInfo & MyInfos = GetChild<KaxInfo>(FileSegment);
		KaxSegmentFilename & MyKaxSegmentFilename = GetChild<KaxSegmentFilename>(MyInfos);
		*static_cast<EbmlUnicodeString *>(&MyKaxSegmentFilename ) = input_Filename.c_str();

		//Add timecode scale
		KaxTimecodeScale & TimeScale = GetChild<KaxTimecodeScale>(MyInfos);
		*(static_cast<EbmlUInteger *>(&TimeScale)) = TIMECODE_SCALE;

		KaxDuration & Duration = GetChild<KaxDuration>(MyInfos);
		*(static_cast<EbmlFloat *>(&Duration)) = Duration_total;

		MyInfos.Render(output_file);

		KaxTracks & MyTracks = GetChild<KaxTracks>(FileSegment);

		//Fill in track params
		MyTrack1 = &GetChild<KaxTrackEntry>(MyTracks);
		KaxTrackNumber & MyTrack1Number = GetChild<KaxTrackNumber>(*MyTrack1);
		*(static_cast<EbmlUInteger *>(&MyTrack1Number)) = 1;
		*(static_cast<EbmlUInteger *>(&GetChild<KaxTrackType>(*MyTrack1))) = track_audio;
		
		KaxTrackUID & MyKaxTrack1UID = GetChild<KaxTrackUID>(*MyTrack1);
		*(static_cast<EbmlUInteger *>(&MyKaxTrack1UID)) = track_uid;

		KaxCodecID & MyTrack1CodecID = GetChild<KaxCodecID>(*MyTrack1);
		if(Layer == 2){//MP3
				MyTrack1CodecID.CopyBuffer((binary*)"A_MPEG/L3",countof("A_MPEG/L3"));
		}else if(Layer == 1){//MP2
				MyTrack1CodecID.CopyBuffer((binary*)"A_MPEG/L2",countof("A_MPEG/L2"));
		}else{ //MP1 or unknown    
				MyTrack1CodecID.CopyBuffer((binary*)"A_MPEG/L1",countof("A_MPEG/L1"));    
		}

		KaxTrackAudio & MyTrack1Audio = GetChild<KaxTrackAudio>(*MyTrack1);

		KaxAudioSamplingFreq & MyTrack1Freq = GetChild<KaxAudioSamplingFreq>(MyTrack1Audio);
		*(static_cast<EbmlFloat *>(&MyTrack1Freq)) = (float)SamplingRate;

		MyTrack1Freq.ValidateSize();

		KaxAudioChannels & MyTrack1Channels = GetChild<KaxAudioChannels>(MyTrack1Audio);
		*(static_cast<EbmlUInteger *>(&MyTrack1Channels)) = Channels;

		uint64 TrackSize = MyTracks.Render(output_file);
		MySeekHead.IndexThis(MyTracks, FileSegment);

		printf("Matroska File created...\n");
		return true;
	}catch (exception & Ex)
	{
		cout << Ex.what() << endl;        
	}
	return false;
}

bool MPA2MKA::AddFrame(StdIOCallback & output_file, KaxTrackEntry *TheTrack, binary* buffer, uint64 bufferLength, uint64 timecode){
  try{       
		bool clusterChange = ChangeClusters(output_file, timecode);
    if((!currentBlockGroup) || currentBlockGroupFull|| clusterChange) {        
      //AllCues.AddBlockGroup(*currentBlockGroup);    
			currentBlockGroup = &clust->GetNewBlock();
			currentBlockGroup->SetBlockDuration((timecode-currentClusterBlockTimecode) * TIMECODE_SCALE);
			currentClusterBlockTimecode = timecode;
    }            

		data = new SimpleDataBuffer(buffer, bufferLength, 0);    
		if (noLacing) {
			currentBlockGroupFull = currentBlockGroup->AddFrame(*TheTrack, timecode * TIMECODE_SCALE, *data);
		}else {
			currentBlockGroupFull = !currentBlockGroup->AddFrame(*TheTrack,timecode * TIMECODE_SCALE, *data);
		}
  }catch (exception & Ex) {
    cout << Ex.what() << endl;
    return false;
  }
  return true;    
}

bool MPA2MKA::ChangeClusters(StdIOCallback & output_file, uint64 timecode){
  try{
    if(!clust || ((timecode - currentClusterBaseTimecode) >= maxTimePerCluster) || (currentClusterSize >= maxClusterSize)) {
      if(clust){
        clust->Render(output_file, AllCues);
        clust->ReleaseFrames();
				if (currentClusterBaseTimecode == 0) {
					MySeekHead.IndexThis(*clust, FileSegment);
				}
        delete clust;
				delete data;
				//delete frame_buffer;
        currentClusterSize = 0;
#if defined wxUSE_GUI
				wxYield();
#endif
      }
      clust = new KaxCluster();
      clust->SetParent(FileSegment);			
      clust->SetPreviousTimecode(currentClusterBaseTimecode * TIMECODE_SCALE);
      KaxClusterTimecode & MyClusterTimeCode = GetChild<KaxClusterTimecode>(*clust);
      *(static_cast<EbmlUInteger *>(&MyClusterTimeCode)) = timecode;
      currentClusterBaseTimecode = timecode;
      return true;
    }else{
      return false;
    }
  }catch (exception & Ex){
    cout << Ex.what() << endl;
    return false;
  }   
}

void MPA2MKA::CloseMatroskaFile(StdIOCallback & output_file){
	if(clust){
    clust->Render(output_file, AllCues);
    clust->ReleaseFrames();
    delete clust;		
  }
	KaxTags & MyKaxTags = GetChild<KaxTags>(FileSegment);
	//Add info tag for writing app
	KaxWritingApp & MyKaxWritingApp = GetChild<KaxWritingApp>(MyKaxTags);
	*static_cast<EbmlUnicodeString *>(&MyKaxWritingApp) = mpa2mka_app_name;

	KaxMuxingApp & MyKaxMuxingApp = GetChild<KaxMuxingApp>(MyKaxTags);
	*static_cast<EbmlUnicodeString *>(&MyKaxMuxingApp) = "libmatroska";

	KaxTag & MyKaxTag = GetChild<KaxTag>(MyKaxTags);
	KaxTagFile & MyKaxTagFile = GetChild<KaxTagFile>(MyKaxTag);
	*static_cast<EbmlUnicodeString *>(&MyKaxTagFile) = input_Filename.c_str();

	//Fill Tag UID
	KaxTagTrackUID & MyKaxTagTrackUID = GetChild<KaxTagTrackUID>(MyKaxTag);
	*(static_cast<EbmlUInteger *>(&MyKaxTagTrackUID)) = track_uid;

#ifdef HAVE_ID3LIB
	if (bCopyTags) {
		using namespace id3lib;
		//Start ID3 Tag preserving
		ID3Tag *tag;
		char tag_txt[1024];

		tag = ID3Tag_New();
		(void) ID3Tag_Link(tag, input_Filename.c_str());
		printf("\nID3 Tag Infomation: ");
		if (ID3Tag_HasTagType(tag, ID3TT_ID3V1))
			printf("ID3v1 detected, ");
		if (ID3Tag_HasTagType(tag, ID3TT_ID3V2))
			printf("ID3v2 detected, ");
		if (ID3Tag_HasTagType(tag, ID3TT_MUSICMATCH))
			printf("MusicMatch ID3 tag detected, ");
		if (ID3Tag_HasTagType(tag, ID3TT_LYRICS) | ID3Tag_HasTagType(tag, ID3TT_LYRICS3) | ID3Tag_HasTagType(tag, ID3TT_LYRICS3V2))
			printf("Lyrics tag detected, ");

		printf("%i fields found\n", ID3Tag_NumFrames(tag));
		//Start searching for frames and fields
		ID3Frame *frame;
		ID3Field *field;
			
		//Create the empty starting Multi's
		KaxTagMultiComment & MyKaxTagMultiComment = GetChild<KaxTagMultiComment>(MyKaxTag);
		KaxTagMultiIdentifier & MyKaxTagMultiIdentifier = GetChild<KaxTagMultiIdentifier>(MyKaxTag);
		KaxTagMultiTitle & MyKaxTagMultiTitle = GetChild<KaxTagMultiTitle>(MyKaxTag);
		KaxTagMultiEntity & MyKaxTagMultiEntity = GetChild<KaxTagMultiEntity>(MyKaxTag);

		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_COMMENT)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Create a new Multi-Identifier element
				KaxTagMultiComment & MyKaxTagMultiComment_COMMENT = GetNextChild<KaxTagMultiComment>(MyKaxTag, MyKaxTagMultiComment);
				//Set the correct Language, English for now
				KaxTagMultiCommentLanguage & MyKaxTagMultiCommentLanguage_COMMENT = GetChild<KaxTagMultiCommentLanguage>(MyKaxTagMultiComment_COMMENT);
				*static_cast<EbmlString *>(&MyKaxTagMultiCommentLanguage_COMMENT) = "eng";
				//Fill the Name sub-element
				KaxTagMultiCommentComments & MyKaxTagMultiCommentComments_COMMENT = GetChild<KaxTagMultiCommentComments>(MyKaxTagMultiComment_COMMENT);
				*static_cast<EbmlUnicodeString *>(&MyKaxTagMultiCommentComments_COMMENT) = tag_txt;
				//Go ahead and render this tag
				//MyKaxTagMultiComment.Render(output_file);
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_CDID)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Create a new Multi-Identifier element
				KaxTagMultiIdentifier & MyKaxTagMultiIdentifier_CDID = GetNextChild<KaxTagMultiIdentifier>(MyKaxTag, MyKaxTagMultiIdentifier);
				//Set the correct Type
				KaxTagMultiIdentifierType & MyKaxTagMultiIdentifierType_CDID = GetChild<KaxTagMultiIdentifierType>(MyKaxTagMultiIdentifier_CDID);
				*static_cast<EbmlUInteger *>(&MyKaxTagMultiIdentifierType_CDID) = KaxTagMultiIdentifierType_CDIdentifier;
				//Fill the Name sub-element
				KaxTagMultiIdentifierString & MyKaxTagMultiIdentifierString_CDID = GetChild<KaxTagMultiIdentifierString>(MyKaxTagMultiIdentifier_CDID);
				*static_cast<EbmlUnicodeString *>(&MyKaxTagMultiIdentifierString_CDID) = tag_txt;
				//Go ahead and render this tag
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_ALBUM)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Title
				KaxTagMultiTitle & MyKaxTagMultiTitle_ALBUM = GetNextChild<KaxTagMultiTitle>(MyKaxTag, MyKaxTagMultiTitle);

				KaxTagMultiTitleType & MyKaxTagMultiTitleType_ALBUM = GetChild<KaxTagMultiTitleType>(MyKaxTagMultiTitle_ALBUM);
				*static_cast<EbmlUInteger *>(&MyKaxTagMultiTitleType_ALBUM) = KaxTagMultiTitleType_AlbumMovieShowTitle;

				KaxTagMultiTitleName & MyKaxTagMultiTitleName_ALBUM = GetChild<KaxTagMultiTitleName>(MyKaxTagMultiTitle_ALBUM);
				*static_cast<EbmlUnicodeString *>(&MyKaxTagMultiTitleName_ALBUM) = tag_txt;
			}
		}			
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_TITLE)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Title
				KaxTagMultiTitle & MyKaxTagMultiTitle_TITLE = GetNextChild<KaxTagMultiTitle>(MyKaxTag, MyKaxTagMultiTitle);

				KaxTagMultiTitleType & MyKaxTagMultiTitleType_TITLE = GetChild<KaxTagMultiTitleType>(MyKaxTagMultiTitle_TITLE);
				*static_cast<EbmlUInteger *>(&MyKaxTagMultiTitleType_TITLE) = KaxTagMultiTitleType_TrackTitle;

				KaxTagMultiTitleName & MyKaxTagMultiTitleName_TITLE = GetChild<KaxTagMultiTitleName>(MyKaxTagMultiTitle_TITLE);
				*static_cast<EbmlUnicodeString *>(&MyKaxTagMultiTitleName_TITLE) = tag_txt;
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_LEADARTIST)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Tag in the mka
				KaxTagMultiEntity & MyKaxTagMultiEntity_LEADARTIST = GetNextChild<KaxTagMultiEntity>(MyKaxTag, MyKaxTagMultiEntity);

				KaxTagMultiEntityType & MyKaxTagMultiEntityType_LEADARTIST = GetChild<KaxTagMultiEntityType>(MyKaxTagMultiEntity_LEADARTIST);
				*static_cast<EbmlUInteger *>(&MyKaxTagMultiEntityType_LEADARTIST) = KaxTagMultiEntitiesType_LeadPerformerSoloist;

				KaxTagMultiEntityName & MyKaxTagMultiEntityName_LEADARTIST = GetChild<KaxTagMultiEntityName>(MyKaxTagMultiEntity_LEADARTIST);
				*static_cast<EbmlUnicodeString *>(&MyKaxTagMultiEntityName_LEADARTIST) = tag_txt;
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_COMPOSER)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Tag in the mka
				KaxTagMultiEntity & MyKaxTagMultiEntity_COMPOSER = GetNextChild<KaxTagMultiEntity>(MyKaxTag, MyKaxTagMultiEntity);

				KaxTagMultiEntityType & MyKaxTagMultiEntityType_COMPOSER = GetChild<KaxTagMultiEntityType>(MyKaxTagMultiEntity_COMPOSER);
				*static_cast<EbmlUInteger *>(&MyKaxTagMultiEntityType_COMPOSER) = KaxTagMultiEntitiesType_Composer;

				KaxTagMultiEntityName & MyKaxTagMultiEntityName_COMPOSER = GetChild<KaxTagMultiEntityName>(MyKaxTagMultiEntity_COMPOSER);
				*static_cast<EbmlUnicodeString *>(&MyKaxTagMultiEntityName_COMPOSER) = tag_txt;
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_ENCODEDBY)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Tag in the mka
				KaxTagMultiEntity & MyKaxTagMultiEntity_ENCODEDBY = GetNextChild<KaxTagMultiEntity>(MyKaxTag, MyKaxTagMultiEntity);

				KaxTagMultiEntityType & MyKaxTagMultiEntityType_ENCODEDBY = GetChild<KaxTagMultiEntityType>(MyKaxTagMultiEntity_ENCODEDBY);
				*static_cast<EbmlUInteger *>(&MyKaxTagMultiEntityType_ENCODEDBY) = KaxTagMultiEntitiesType_EncodedBy;

				KaxTagMultiEntityName & MyKaxTagMultiEntityName_ENCODEDBY = GetChild<KaxTagMultiEntityName>(MyKaxTagMultiEntity_ENCODEDBY);
				*static_cast<EbmlUnicodeString *>(&MyKaxTagMultiEntityName_ENCODEDBY) = tag_txt;
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_CONTENTTYPE)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Genre
				KaxTagAudioGenre & MyKaxTagAudioGenre = GetChild<KaxTagAudioGenre>(MyKaxTag);
				*static_cast<EbmlString *>(&MyKaxTagAudioGenre) = tag_txt;
				//MyKaxTagAudioGenre.Render(output_file);
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_TRACKNUM)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Disc Track Number
				KaxTagDiscTrack & MyKaxTagDiscTrack = GetChild<KaxTagDiscTrack>(MyKaxTag);
				*static_cast<EbmlUInteger *>(&MyKaxTagDiscTrack) = atol(tag_txt);
				//MyKaxTagDiscTrack.Render(output_file);
			}
		}
		if ((frame = ID3Tag_FindFrameWithID(tag, ID3FID_WWWUSER)) != NULL)
		{
			if ((field = ID3Frame_GetField(frame, ID3FN_TEXT)) != NULL)
			{ 
				(void) ID3Field_GetASCII(field, tag_txt, 1024);
				//Store the ID3 Web URL
				KaxTagUserDefinedURL & MyKaxTagUserDefinedURL = GetChild<KaxTagUserDefinedURL>(MyKaxTag);
				*static_cast<EbmlString *>(&MyKaxTagUserDefinedURL) = tag_txt;
			}
		}
	}
#else
	printf("ID3 Tag support not included\n");
#endif	
	MyKaxTags.Render(output_file);
	
	MySeekHead.IndexThis(MyKaxTags, FileSegment);
	Dummy.ReplaceWith(MySeekHead, output_file);

  printf("File closed.\n");
}
