#include <iostream>
#include <stdio.h>
#include <stdarg.h>
#include <time.h>

#include "StdIOCallback.h"

#include "EbmlHead.h"
#include "EbmlSubHead.h"
#include "EbmlVoid.h"
#include "FileKax.h"
#include "KaxSegment.h"
#include "KaxTracks.h"
#include "KaxTrackEntryData.h"
#include "KaxTrackAudio.h"
#include "KaxTrackVideo.h"
#include "KaxCluster.h"
#include "KaxClusterData.h"
#include "KaxBlock.h"
#include "KaxBlockData.h"
#include "KaxCues.h"
#include "KaxSeekHead.h"
#include "KaxInfo.h"
#include "KaxInfoData.h"
#include "KaxTags.h"
#include "KaxTag.h"
#include "KaxTagMulti.h"
#include "KaxVersion.h"
#include "mpafile.h"

#ifdef HAVE_ID3LIB
namespace id3lib {
#include <id3.h>
};
#endif

#if defined wxUSE_GUI
#include <wx/wx.h>
#endif

using namespace LIBMATROSKA_NAMESPACE;
using namespace LIBEBML_NAMESPACE;
using namespace std;

#define mpa2mka_app_name "mpa2mka v0.3.2"

class MPA2MKA
{
public:
	MPA2MKA();
	~MPA2MKA();
	int ConvertMPA2Matroska(std::string passed_InputFile, std::string passed_OutputFile = "");
	bool SetInputFile(std::string passed_InputFile); //Set the input mpeg audio file
	bool SetOutputFile(std::string passed_OutputFile); //Sets the output Matroska file
	bool SetVerboseModeOn(bool bVerboseOn = false); //Prepare to be blasted with numbers, Default False
	bool SetLacingOff(bool bLacingOff = false); //Turns off Lacing, which increases file sizes, Default False
	bool SetTagCopying(bool bTagCopying = true); //Change is ID3 Tags are copied to the Matroska file, Default True
	bool SetMaxClusterByteSize(uint64 passed_maxClusterSize = 524288); //Sets the largest cluster size in bytes, Ranges 128bytes to 1GB
	bool SetMaxClusterTimeSize(uint32 passed_maxTimePerCluster = 5000); //Sets the largest cluster size in ms, Ranges 10ms to 1hour
#if defined wxUSE_GUI
	void ConvertMPA2Matroska_wxWindows(wxTextCtrl *target_console_output_box, std::string passed_InputFile, std::string passed_OutputFile = "");
	bool SetOutputTextBox_wxWindows(wxTextCtrl *target_console_output_box);
#endif
protected:
	void OutputToBuffer(char *formating_string, ...);
	bool ChangeClusters(StdIOCallback & output_file, uint64 timecode);
	int InitMatroskaOutputFile(StdIOCallback & output_file, uint16 SamplingRate, uint16 Layer, uint16 Channels);
	bool AddFrame(StdIOCallback & output_file, KaxTrackEntry *TheTrack, binary* buffer, uint64 bufferLength, uint64 timecode);
	void CloseMatroskaFile(StdIOCallback & output_file);

	std::string input_Filename;
	std::string input_array[2];
	std::string output_Filename;

	KaxCluster *clust;
	KaxCues AllCues;
	KaxTrackEntry *MyTrack1;
	KaxTrackEntry *MyTrack2;
	KaxBlockGroup *currentBlockGroup;//, *lastBlockGroup;
	KaxSegment FileSegment;
	EbmlVoid Dummy;
	KaxSeekHead MySeekHead;

	KaxInfo MyInfos;
	KaxTitle MyInfosTitle;
	KaxSegmentFilename MyKaxSegmentFilename;
	KaxTags MyKaxTags;
	KaxWritingApp MyKaxWritingApp;

	binary* frame_buffer;
	DataBuffer* data;
	bool currentBlockGroupFull;
	bool noLacing;
	bool bVerbose;
	bool bCopyTags;
	uint32 maxTimePerCluster; //Default Value is 10 seconds in ms
	uint64 maxClusterSize; //Default value is 512kB in bytes
	uint64 currentClusterSize;
	uint64 currentClusterBaseTimecode;
	uint64 currentClusterBlockTimecode;
	float Duration_total;
	uint64 TIMECODE_SCALE;
	uint32 track_uid;
#if defined wxUSE_GUI	
	wxTextCtrl *console_output_box;
#endif
};
