
#include "wav2mka.h"

void usage()
{
  printf("This program converts a RIFF/WAV file into an Matroska file\n\n");
  printf("Usage: wav2mka <input-filename> [output-filename]\n");
  printf("\tif no output file is specified the input filename is appended with .mka\n");
	printf("\tOptions:\r\n\r\n");
	printf("\t\t -v VERBOSE\tDisplay more infomation.\n\n");
	printf("\t\t -nl NO LACING\tTurns off lacing.\n\n");
	printf("\t\t -c SIZE\tSets the max size in kB of a cluster.\n\t\t\t\tDefault is 512kB\n\n");
	printf("\t\t -t TIME\tSets the max time in ms per cluster.\n\t\t\t\tDefault is 10000ms\n\n");
}

int main(int argc,char *argv[])
{
	using namespace std;
	int cl_offset = 0;
	bool bVerbose = false;
	bool noLacing = false;
	uint32 maxTimePerCluster = 5000; //10 seconds
	uint64 maxClusterSize = 512 * 1024; //512kB

	// check for the valid number of parameters
  if (argc < 2 || argc > 8) {
		usage();
		return -1;
  }

	if(argc > 2){ //option may be included
		for(int i = 0; i < 4; i++){
			if((argv[1+cl_offset][0] == '-') && ((argv[1+cl_offset][1] == 'c') || (argv[1+cl_offset][1] == 't') || (argv[1+cl_offset][1] == 'v') || ((argv[1+cl_offset][1] == 'n') && (argv[1+cl_offset][2] == 'l')))){
				if(argv[1+cl_offset][1] == 'c'){
					maxClusterSize = (atoi(argv[2+cl_offset]) * 1024);
					cl_offset+=2;
				}
				if(argv[1+cl_offset][1] == 't'){
					uint32 maxTimePerCluster = atoi(argv[2+cl_offset]);
					cl_offset+=2;
				}
				if(argv[1+cl_offset][1] == 'v'){
					bVerbose = true;
					
					cl_offset+=1;
				}
				if(argv[1+cl_offset][1] == 'n' & argv[1+cl_offset][2] == 'l'){
					noLacing = true;
					
					cl_offset+=1;
				}
				//cl_offset+=2;
			}else{
				break;
			}
		}
	}
  std::string output_Filename;

	if (argc > cl_offset+2) {
    output_Filename = argv[2+cl_offset];
	}else {
    output_Filename.append(argv[1+cl_offset]);
		output_Filename.append(".mka");
	}
	WAV2MKA *new_convert_process;
	new_convert_process = new WAV2MKA();

	new_convert_process->SetVerboseModeOn(bVerbose);
	new_convert_process->SetLacingOff(noLacing);
	new_convert_process->SetMaxClusterTimeSize(maxTimePerCluster);
	new_convert_process->SetMaxClusterByteSize(maxClusterSize);
	cout << new_convert_process->ConvertWAV2Matroska(argv[1+cl_offset], output_Filename);

	return 0;
}