/*
	Copyright (C) 2003, Jory Stone <jcsston @ toughguy.net>
	Copyright (C) 2003, John Cannon <spyder482 @ yahoo.com>
	Copyright (C) 2002, Steve Lhomme <steve.lhomme @ free.fr>
	This file is part of wav2mka.

	Wav2mka is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	Wav2mka is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with Foobar; if not, write to the Free Software
	Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
/**
	\file wav2mka.cpp
	Convert WAV to Matroska files
	\todo remove bad frames
*/

#include "wav2mka.h"
using namespace LIBMATROSKA_NAMESPACE;
using namespace LIBEBML_NAMESPACE;
using namespace std;

#if defined wxUSE_GUI
#define printf OutputToBuffer
void WAV2MKA::ConvertWAV2Matroska_wxWindows(wxTextCtrl *target_console_output_box, std::string passed_InputFile, std::string passed_OutputFile) {
	console_output_box = target_console_output_box;
	ConvertWAV2Matroska(passed_InputFile, passed_OutputFile);
	return;
}
#endif

void WAV2MKA::OutputToBuffer(char *formating_string, ...) {
	va_list argptr;

	//Get a pointer to the list of arg
	va_start(argptr, formating_string);	
#if defined wxUSE_GUI 
	//Redirect the output to a wxWindow wxTextCtrl
	wxString formated_text;
	formated_text.PrintfV(wxString(formating_string, wxConvUTF8).c_str(), argptr);
	console_output_box->AppendText(formated_text);
	//Give the processor some relief
	wxYield();
#else
	//Just let the function call pass through unaffected
	vprintf(formating_string, argptr);	
#endif
}


WAV2MKA::WAV2MKA() {
	currentBlockGroup = NULL;
	clust = NULL;

	currentBlockGroupFull = false;
	noLacing = false;
	bVerbose = false;
	maxTimePerCluster = 5000;	 //10 seconds
	maxClusterSize = 512 * 1024; //512kB
	currentClusterSize = 0;
	currentClusterBaseTimecode = 0;
	currentClusterBlockTimecode = 0;
	TIMECODE_SCALE = 1000000;
};

bool WAV2MKA::ChangeClusters(StdIOCallback & output_file, uint64 timecode) {
	try {
		if(!clust || ((timecode - currentClusterBaseTimecode) >= maxTimePerCluster) || (currentClusterSize >= maxClusterSize)) {
			if(clust) {
				//clust->SetPreviousTimecode(currentClusterBaseTimecode);
				clust->Render(output_file, AllCues);
				clust->ReleaseFrames();
				delete clust;
				currentClusterSize = 0;
#if defined wxUSE_GUI
				wxYield();
#endif
			}
			clust = new KaxCluster();
			clust->SetParent(FileSegment);
			clust->SetPreviousTimecode(currentClusterBaseTimecode * TIMECODE_SCALE);
			KaxClusterTimecode & MyClusterTimeCode = GetChild<KaxClusterTimecode>(*clust);
			*(static_cast<EbmlUInteger *>(&MyClusterTimeCode)) = timecode;
			currentClusterBaseTimecode = timecode;
			return true;
		}
		else {
			return false;
		}
	}
	catch (exception & Ex) {
		cout << Ex.what() << endl;
		return false;
	}
}


bool WAV2MKA::SetInputFile(std::string passed_InputFile) {
	input_Filename = passed_InputFile;
	return true;
};

bool WAV2MKA::SetOutputFile(std::string passed_OutputFile) {
	output_Filename = passed_OutputFile;
	return true;
};

bool WAV2MKA::SetVerboseModeOn(bool bVerboseOn) {
	bVerbose = bVerboseOn;
	return true;
};

bool WAV2MKA::SetLacingOff(bool bLacingOff) {
	noLacing = bLacingOff;
	return true;
};

bool WAV2MKA::SetMaxClusterByteSize(uint64 passed_maxClusterSize) {
	//The allowed range is 128 bytes to 1073741824 bytes or 1GB
	if ((passed_maxClusterSize > 128) && (passed_maxClusterSize < 1073741824)) {
		maxClusterSize = passed_maxClusterSize;
		return true;
	}
	else {
		//Assign the default value
		maxClusterSize = 512 * 1024;
		return false;
	}
};

bool WAV2MKA::SetMaxClusterTimeSize(uint32 passed_maxTimePerCluster) {
	//The allowed range is 10ms to 3600000ms or 1 hour
	if ((passed_maxTimePerCluster > 10) && (passed_maxTimePerCluster < 3600000)) {
		maxTimePerCluster = passed_maxTimePerCluster;
		return true;
	}
	else {
		//Assign the default value, 5 seconds
		maxTimePerCluster = 5000;
		return false;
	}
};

int WAV2MKA::InitMatroskaOutputFile(StdIOCallback & output_file, FMT_chunk &fmtChunk) {
	try {
		// write output file
		EbmlHead FileHead;

		EDocType & MyDocType = GetChild<EDocType>(FileHead);
		*static_cast<EbmlString *>(&MyDocType) = "matroska";

		EDocTypeVersion & MyDocTypeVer = GetChild<EDocTypeVersion>(FileHead);
		*(static_cast<EbmlUInteger *>(&MyDocTypeVer)) = 1;

		EDocTypeReadVersion & MyDocTypeReadVer = GetChild<EDocTypeReadVersion>(FileHead);
		*(static_cast<EbmlUInteger *>(&MyDocTypeReadVer)) = 1;

		FileHead.Render(output_file);

		uint64 SegmentSize = FileSegment.Render(output_file);

		KaxTags & MyKaxTags = GetChild<KaxTags>(FileSegment);
		KaxTag & MyKaxTag = GetChild<KaxTag>(MyKaxTags);

		srand((unsigned)time(NULL));
		uint32 track_uid = rand();
		KaxTagTrackUID & MyKaxTagTrackUID = GetChild<KaxTagTrackUID>(MyKaxTag);
		*(static_cast<EbmlUInteger *>(&MyKaxTagTrackUID)) = track_uid;

		//if (broadcast_audio_extension wav_tag_chunk) {
			//Add the tags
		//}

		//Add info tag for writing app
		KaxWritingApp & MyKaxWritingApp = GetChild<KaxWritingApp>(MyKaxTags);
		*static_cast<EbmlUnicodeString *>(&MyKaxWritingApp) = wav2mka_app_name;
		MyKaxTags.Render(output_file);

		KaxInfo & MyInfos = GetChild<KaxInfo>(FileSegment);
		KaxSegmentFilename & MyKaxSegmentFilename = GetChild<KaxSegmentFilename>(MyInfos);
		*static_cast<EbmlUnicodeString *>(&MyKaxSegmentFilename ) = input_Filename.c_str();

		KaxTimecodeScale & TimeScale = GetChild<KaxTimecodeScale>(MyInfos);
		*(static_cast<EbmlUInteger *>(&TimeScale)) = TIMECODE_SCALE;

		KaxDuration & Duration = GetChild<KaxDuration>(MyInfos);
		*(static_cast<EbmlFloat *>(&Duration)) = 0.0;

		MyInfos.Render(output_file);

		KaxTracks & MyTracks = GetChild<KaxTracks>(FileSegment);

		//Fill in track params
		MyTrack1 = &GetChild<KaxTrackEntry>(MyTracks);
		KaxTrackNumber & MyTrack1Number = GetChild<KaxTrackNumber>(*MyTrack1);
		*(static_cast<EbmlUInteger *>(&MyTrack1Number)) = 1;
		*(static_cast<EbmlUInteger *>(&GetChild<KaxTrackType>(*MyTrack1))) = track_audio;
		
		KaxTrackUID & MyKaxTrackUID = GetChild<KaxTrackUID>(*MyTrack1);
		*(static_cast<EbmlUInteger *>(&MyKaxTrackUID)) = track_uid;

		KaxCodecID & MyTrack1CodecID = GetChild<KaxCodecID>(*MyTrack1);
		//A_PCM/INT/BIG : audio, PCM Big Endian Integer 24 bits
		//A_PCM/INT/LIT : audio, Little Endian Integer
		//A_MS/ACM : audio, Microsoft(TM) Audio Codec Manager (ACM)
		WAVEFORMATEX_ *acm_wav = fmtChunk.GetWAVEFORMATEX();
		
		if ((acm_wav->wFormatTag[0] == 1) && (acm_wav->wFormatTag[1] == 0))
		{
			//PCM
			MyTrack1CodecID.CopyBuffer((binary*)"A_PCM/INT/LIT",countof("A_PCM/INT/LIT"));
		} else {
			//Other, store as MS/ACM
			MyTrack1CodecID.CopyBuffer((binary*)"A_MS/ACM",countof("A_MS/ACM"));
			
			KaxCodecPrivate & MyTrack1CodecPrivate = GetChild<KaxCodecPrivate>(*MyTrack1);
			binary *cpriv = (binary *)malloc(sizeof(WAVEFORMATEX_)+acm_wav->cbSize);
			memcpy(cpriv, acm_wav, sizeof(WAVEFORMATEX_)+acm_wav->cbSize);
			MyTrack1CodecPrivate.SetBuffer((binary*)cpriv, sizeof(WAVEFORMATEX_)+acm_wav->cbSize);
		}
		

		KaxTrackAudio & MyTrack1Audio = GetChild<KaxTrackAudio>(*MyTrack1);

		KaxAudioSamplingFreq & MyTrack1Freq = GetChild<KaxAudioSamplingFreq>(MyTrack1Audio);
		*(static_cast<EbmlFloat *>(&MyTrack1Freq)) = (float)fmtChunk.SampleRate();

		MyTrack1Freq.ValidateSize();

		KaxAudioChannels & MyTrack1Channels = GetChild<KaxAudioChannels>(MyTrack1Audio);
		*(static_cast<EbmlUInteger *>(&MyTrack1Channels)) = fmtChunk.ChannelPerSubtrack();

		KaxAudioBitDepth & MyTrack1BitDepth = GetChild<KaxAudioBitDepth>(MyTrack1Audio);
		*(static_cast<EbmlUInteger *>(&MyTrack1BitDepth)) = fmtChunk.BitDepth();

		uint64 TrackSize = MyTracks.Render(output_file);
		printf("Matroska File created...\n");
		return true;
	}catch (exception & Ex){
		cout << Ex.what() << endl;

	}
	return false;
}


bool WAV2MKA::AddFrame(StdIOCallback & output_file, binary* buffer, uint64 bufferLength, uint64 timecode) {
	try {
		bool clusterChange = ChangeClusters(output_file, timecode);
		if((!currentBlockGroup) || currentBlockGroupFull || clusterChange) {
			currentBlockGroup = &clust->GetNewBlock();
			currentBlockGroup->SetBlockDuration((timecode-currentClusterBlockTimecode) * TIMECODE_SCALE);
			currentClusterBlockTimecode = timecode;
		}
		binary* buffer2 = new binary[bufferLength];
		memcpy(buffer2, (const binary *)buffer, bufferLength);
		DataBuffer* data = new SimpleDataBuffer(buffer2, bufferLength, 0);
		currentBlockGroupFull = !currentBlockGroup->AddFrame(*MyTrack1, (timecode * TIMECODE_SCALE), *data);
	}
	catch (exception & Ex) {
		cout << Ex.what() << endl;
		return false;
	}
	return true;
}


void WAV2MKA::CloseMatroskaFile(StdIOCallback & output_file) {
	if(clust) {
		clust->Render(output_file, AllCues);
		clust->ReleaseFrames();
		delete clust;
	}
	printf("File closed.\n");
}


int WAV2MKA::ConvertWAV2Matroska(std::string passed_InputFile, std::string passed_OutputFile) {
	using namespace M2W_NAMESPACE;
	//Store values
	input_Filename = passed_InputFile;

	if (passed_OutputFile == "")
		output_Filename = passed_InputFile + ".mka";
	else
		output_Filename = passed_OutputFile;

	try {
		printf("%s - (c) 2003 Jory Stone\n", wav2mka_app_name);
		printf("Portions (c) 2003 John Cannon\n");
		printf("Portions (c) 2002 Steve Lhomme\n");
		printf("Under the GNU GPL License\n\n");

		if (bVerbose)
			printf("In Verbose Mode, prepare to be blasted with numbers\n");
		if (noLacing)
			printf("Lacing turned off\n");

		WAV_file input_File(input_Filename);

		if (!input_File.Open(RIFF_file::RIFF_READ)) {
			printf("Impossible to read the file '%s'\n", input_Filename.c_str());
			return -1;
		}

		// open the output file
		StdIOCallback output_file(output_Filename.c_str(), MODE_CREATE);

		// prepare to read the whole file for statistics
		if (bVerbose) {
			printf("Reading statistics in %s\n", input_Filename.c_str());
		}

		input_File.ValidateHeader();

		FMT_chunk *fmtChunk = NULL;
		DATA_chunk *dataChunk = NULL;
		BEXT_chunk *bextChunk = NULL;
		LIST_chunk *listChunk = NULL;
		LIST_Info *listInfo = NULL;
		while (!dataChunk && input_File.FindNextChunk()) {
			switch(input_File.ChunkType()) {
				case WAV_file::DATA_CHUNK:
					dataChunk = new DATA_chunk(input_File.AcquireChunk());
					input_File.SetDataChunk(dataChunk);
					break;
				case WAV_file::FORMAT_CHUNK:
					fmtChunk = new FMT_chunk(input_File.AcquireChunk());
					input_File.SetFormatChunk(fmtChunk);
					fmtChunk->Read(input_File.IOfile());
					break;
				case WAV_file::BROADCAST_CHUNK:
					bextChunk = new BEXT_chunk(input_File.AcquireChunk());
					bextChunk->Read(input_File.IOfile());
					break;
				case WAV_file::LIST_CHUNK:
					listChunk = new LIST_chunk(input_File.AcquireChunk());
					switch(listChunk->FindType(input_File.IOfile())) {
						case LIST_chunk::LIST_INFO:
							listInfo = new LIST_Info;
							if (listInfo == NULL) {
								printf("No more space to save LIST INFO\n");
								listChunk->SkipData(input_File.IOfile());
							}
							else {
								if (!listInfo->Read(input_File.IOfile())) {
									listChunk->SkipData(input_File.IOfile());
									delete listInfo;
									listInfo = NULL;
								}
							}
							break;
						default:
							listChunk->SkipData(input_File.IOfile());
							break;
					}
					delete listChunk;
					listChunk = NULL;
					break;
				default:
					// unknown chunk, skip data
					RIFF_chunk * Unknown = input_File.AcquireChunk();
					Unknown->SkipData(input_File.IOfile());
					break;
			}
		}

		if (!dataChunk) {
			printf("Impossible to find the data part in '%s'\n", input_Filename.c_str());
			return -1;
		}
		if (!fmtChunk) {
			printf("Impossible to find the format part in '%s'\n", input_Filename.c_str());
			return -1;
		}

		printf("Maximum Cluster size is %dkB\n", (maxClusterSize/1024));
		printf("Maximum time per cluster is %dms\n", maxTimePerCluster);

		
		InitMatroskaOutputFile(output_file, *fmtChunk);
		ChangeClusters(output_file, 0);

		// read data from the input file and put them in the output file
		const uint32 maxRead = 5000;
		binary* DataBuffer = new binary[maxRead];;
		uint32 timecode = input_File.GetCurrentTimecode();
		uint32 readSize = input_File.ReadData(DataBuffer, maxRead);
		while (readSize != 0) {
			AddFrame(output_file, DataBuffer, readSize, timecode);
			currentClusterSize += readSize;
			timecode = input_File.GetCurrentTimecode();
			readSize = input_File.ReadData(DataBuffer, maxRead);
		}
		timecode = input_File.GetCurrentTimecode();

		CloseMatroskaFile(output_file);
		output_file.close();

		if (fmtChunk != NULL)
			delete fmtChunk;

		if (dataChunk != NULL)
			delete dataChunk;

		if (bextChunk != NULL)
			delete bextChunk;

		if (listChunk != NULL)
			delete listChunk;

		if (listInfo != NULL)
			delete listInfo;
		printf("\n");
		return 0;

	}
	catch (exception & Ex) {
		cout << Ex.what() << endl;
		return -1;
	}
}
